!*************************************************************************
! This file is part of
!
!       HiggsBounds 2.0.0 (fotran 77 version)
!
! by Philip Bechtle, Oliver Brein, Sven Heinemyer, Georg Weiglein
!    and Karina E. Williams.
!
!  Journal Reference: e-Print: arXiv:0811.4169 [hep-ph], accepted by CPC.
!  Web site: http://www.ippp.dur.ac.uk/HiggsBounds
!
! 1/8/2010
!*************************************************************************


!************************************************************
!      NOTES
!************************************************************
! LEP data files 
! * Program works out which channel is predicted to have the most power to
! exclude the parameter point, then checks the experimental result for that
! point only


!***********************************************************
	subroutine initialize_LEP_tables()
!***********************************************************
! remark: S95_1d(1,..) or S95_2d(1,..) are the predicted limits tables
!         S95_1d(2,..) or S95_2d(2,..) are the observed limits tables
	implicit none
	integer i,j,m,n,imax,jmax,ntot
	integer Sneut_tab,Schar_tab,S_tab,Dneut_tab,index
	real*8 mh1dummy,mh2dummy,dummy

#include "input_cb.h"
#include "tables-indexing-lep_cb.h"
#include "tables-data-lep_cb.h"

	write(*,*)'LEP-Constraints: initialization of data tables'

c intitialize counters for switched-on LEP tables
	Sneut_tab=0
	Schar_tab=0
	S_tab=0
	Dneut_tab=0
	index=0
	
	write(*,*)
        write(*,*)'single neutral Higgs searches:'

#ifdef LEPhZ_bbZ
	index=index+1
	lep_publd(index)=.TRUE.
	nLEPhZ_bbZ=index

	Sneut_tab=Sneut_tab+1
	S_tab=S_tab+1
	id_Sneut(Sneut_tab)=index
	Sid_Sneut(Sneut_tab)=S_tab
	Sneut_id(index)=Sneut_tab
! initialize table 14b
	write(*,*)'reading table 14b (EPJC 46(2006)547)'

! information about how LEP data is stored in file:	 
	Mhmin(S_tab)=12.0D0
	Mhmax(S_tab)=120.0D0
	sep(S_tab)=0.5D0

	ntot=nint((Mhmax(S_tab)-Mhmin(S_tab))/sep(S_tab))+1
! ntot is expected to be 217. 

	open(10,file=INSTDIR//"/LEPtables/lep210_hbb.txt")

! first 5 lines are not needed	 
        do i=1,5
	  read(10,*)  	 
        end do	 	
		 
        do i=1,ntot
	  read(10,*)Mh1dummy,S95_1d(2,S_tab,i),S95_1d(1,S_tab,i)
! remark: S95_1d(1,..) is the predicted limits table
!         S95_1d(2,..) is the observed limits table
        enddo

        close(10)	 	
#endif	

#ifdef LEPhZ_tautauZ
        index=index+1
        lep_publd(index)=.TRUE. 
	nLEPhZ_tautauZ=index

	Sneut_tab=Sneut_tab+1
        S_tab=S_tab+1
	id_Sneut(Sneut_tab)=index
	Sid_Sneut(Sneut_tab)=S_tab
        Sneut_id(index)=Sneut_tab

! initialize table 14c
	write(*,*)'reading table 14c (EPJC 46(2006)547)'

! information about how LEP data is stored in file:	 
	Mhmin(S_tab)=4.0D0
	Mhmax(S_tab)=120.0D0
	sep(S_tab)=0.5D0

	ntot=nint((Mhmax(S_tab)-Mhmin(S_tab))/sep(S_tab))+1
! ntot is expected to be 233. 

	open(10,file=INSTDIR//"/LEPtables/lep210_htt_interpol.txt")

! first 5 lines are not needed	 
        do i=1,5
	  read(10,*)  	 
        end do	 	
		 
        do i=1,ntot
          read(10,*)Mh1dummy,S95_1d(2,S_tab,i),S95_1d(1,S_tab,i)
        enddo

        close(10)	 	
#endif	






#ifdef LEPhZ_anyZ
        index=index+1
        lep_publd(index)=.TRUE. 
	nLEPhZ_anyZ=index

	Sneut_tab=Sneut_tab+1
        S_tab=S_tab+1
        id_Sneut(Sneut_tab)=index
	Sid_Sneut(Sneut_tab)=S_tab
        Sneut_id(index)=Sneut_tab

! new 1) channel e+ e- -> hZ, h -> anything
        write(*,*)'reading table for analysis: e+ e- -> hZ, h -> anything (OPAL, EPJC 27(2003)311)'

! information about how LEP data is stored in file:	 
	Mhmin(S_tab)=1.0D0
	Mhmax(S_tab)=100.0D0
	sep(S_tab)=1D0

	ntot=nint((Mhmax(S_tab)-Mhmin(S_tab))/sep(S_tab))+1
! ntot is expected to be 100. 

	open(10,file=INSTDIR//"/LEPtables/lep_s95_ee_hZ_anythingZ.txt")

! first 5 lines are not needed	 
        do i=1,5
	  read(10,*)  	 
        end do	 	
		 
        do i=1,ntot
          read(10,*)Mh1dummy,S95_1d(1,S_tab,i),S95_1d(2,S_tab,i)
        enddo

        close(10)	 	
#endif


#ifdef LEPhZ_invisZ
        index=index+1
        lep_publd(index)=.TRUE. 
	nLEPhZ_invisZ=index	

	Sneut_tab=Sneut_tab+1
        S_tab=S_tab+1
        id_Sneut(Sneut_tab)=index
	Sid_Sneut(Sneut_tab)=S_tab
        Sneut_id(index)=Sneut_tab

! new 4) channel e+ e- -> hZ, h -> invisible
	write(*,*)'reading table for analysis: e+ e- -> hZ, h -> invisible (LEP, hep-ex/0107032)'

! information about how LEP data is stored in file:	 
	Mhmin(S_tab)=90D0
	Mhmax(S_tab)=118D0
	sep(S_tab)=1D0

	ntot=nint((Mhmax(S_tab)-Mhmin(S_tab))/sep(S_tab))+1
! ntot is expected to be 29. 

	open(10,file=INSTDIR//"/LEPtables/lep_s95_ee_hZ_invisZ.txt")

! first 5 lines are not needed	 
        do i=1,5
	  read(10,*)  	 
        end do	 	
		 
        do i=1,ntot
	  read(10,*) Mh1dummy,S95_1d(2,S_tab,i),S95_1d(1,S_tab,i)
        enddo

        close(10)	 		
#endif



#ifdef LEPhZ_gamgamZ
	if(whichexpt .eq. 'onlyP') then
	nLEPhZ_gamgamZ=-1
	else

        index=index+1
	nLEPhZ_gamgamZ=index	

	Sneut_tab=Sneut_tab+1
        S_tab=S_tab+1
        id_Sneut(Sneut_tab)=index
	Sid_Sneut(Sneut_tab)=S_tab
        Sneut_id(index)=Sneut_tab

! new 2) channel e+ e- -> hZ, h -> gamma gamma
        write(*,*)'reading table for analysis: e+ e- -> hZ, h -> gamma gamma (LHWG note 2002-02)'

! information about how LEP data is stored in file:	 
	Mhmin(S_tab)=20D0
	Mhmax(S_tab)=116.0D0
	sep(S_tab)=2D0

	ntot=nint((Mhmax(S_tab)-Mhmin(S_tab))/sep(S_tab))+1
! ntot is expected to be 97. 

	open(10,file=INSTDIR//"/LEPtables/lep_s95_ee_hZ_gamgamZ.txt")

! first 5 lines are not needed	 
        do i=1,5
	  read(10,*)  	 
        end do	 	
		 
        do i=1,ntot
          read(10,*)Mh1dummy,S95_1d(2,S_tab,i),S95_1d(1,S_tab,i)
        enddo

        close(10)	 	
	endif
#endif


#ifdef LEPhZ_jjZ
	if(whichexpt .eq. 'onlyP') then
	nLEPhZ_jjZ=-1
	else

        index=index+1
	nLEPhZ_jjZ=index	

	Sneut_tab=Sneut_tab+1
        S_tab=S_tab+1
        id_Sneut(Sneut_tab)=index
	Sid_Sneut(Sneut_tab)=S_tab
        Sneut_id(index)=Sneut_tab

! new 3) channel e+ e- -> hZ, h -> hadrons (i.e. q q-bar, g g)
	write(*,*)'reading table for analysis: e+ e- -> hZ, h -> hadrons (LEP combined limit)'
!Philip (23 June 2009): combination of :
! J. Abdallah {\it et al.}  [DELPHI Collaboration], EPJC 44(2005)147 [hep-ex/0510022];
! A. Heister {\it et al.}  [ALEPH Collaboration], PLB 544(2002)25 [hep-ex/0205055];
! G. Abbiendi {\it et al.}  [OPAL Collaboration], PLB 597(2004)11 [hep-ex/0312042];
! G. Abbiendi {\it et al.}  [OPAL Collaboration], EPJC 40(2005)317 [hep-ex/0408097].


! information about how LEP data is stored in file:	 
	Mhmin(S_tab)=20D0
	Mhmax(S_tab)=128.6D0
	sep(S_tab)=0.1D0

	ntot=nint((Mhmax(S_tab)-Mhmin(S_tab))/sep(S_tab))+1
! ntot is expected to be 1087. 

	open(10,file=INSTDIR//"/LEPtables/lep_s95_ee_hZ_jjZ.txt")

! first 3 lines are not needed	 
        do i=1,3
	  read(10,*)  	 
        end do	 	
		 
        do i=1,ntot
	  read(10,*) Mh1dummy,dummy,S95_1d(2,S_tab,i),S95_1d(1,S_tab,i)
        enddo


        close(10)	 		
	endif
#endif




#ifdef DELPHIbbh_4b
        index=index+1
        lep_publd(index)=.TRUE.
	nDELPHIbbh_4b=index	

	Sneut_tab=Sneut_tab+1
        S_tab=S_tab+1
        id_Sneut(Sneut_tab)=index
	Sid_Sneut(Sneut_tab)=S_tab
        Sneut_id(index)=Sneut_tab

! new 5) channel e+ e- -> bbh -> bbbb
	write(*,*)'reading table for analysis: e+ e- -> bbh -> 4b DELPHI (hep-ex/0410017)'

! information about how LEP data is stored in file:	 
	Mhmin(S_tab)=12D0
	Mhmax(S_tab)=50D0
	sep(S_tab)=1D0

	ntot=nint((Mhmax(S_tab)-Mhmin(S_tab))/sep(S_tab))+1
! ntot is expected to be 39. 

	open(10,file=INSTDIR//"/LEPtables/Delphi_yuk_h_bbbb.txt")

! first 5 lines are not needed	 
        do i=1,5
	  read(10,*)  	 
        end do	 	
		 
        do i=1,ntot
	  read(10,*) Mh1dummy,S95_1d(2,S_tab,i),S95_1d(1,S_tab,i)
        enddo

        close(10)	 		
#endif


#ifdef DELPHIbba_4b
        index=index+1
        lep_publd(index)=.TRUE.
	nDELPHIbba_4b=index	

	Sneut_tab=Sneut_tab+1
        S_tab=S_tab+1
        id_Sneut(Sneut_tab)=index
	Sid_Sneut(Sneut_tab)=S_tab
        Sneut_id(index)=Sneut_tab

! new 6) channel e+ e- -> bba -> bbbb
	write(*,*)'reading table for analysis: e+ e- -> bba -> 4b DELPHI (hep-ex/0410017)'

! information about how LEP data is stored in file:	 
	Mhmin(S_tab)=12D0
	Mhmax(S_tab)=50D0
	sep(S_tab)=1D0

	ntot=nint((Mhmax(S_tab)-Mhmin(S_tab))/sep(S_tab))+1
! ntot is expected to be 39. 

	open(10,file=INSTDIR//"/LEPtables/Delphi_yuk_a_bbbb.txt")

! first 5 lines are not needed	 
        do i=1,5
	  read(10,*)  	 
        end do	 	
		 
        do i=1,ntot
	  read(10,*) Mh1dummy,S95_1d(2,S_tab,i),S95_1d(1,S_tab,i)
        enddo

        close(10)	 		
#endif


! new 7) channel e+ e- -> bbh -> 2b2tau
#ifdef DELPHIbbh_2b2tau
        index=index+1
        lep_publd(index)=.TRUE.
	nDELPHIbbh_2b2tau=index	

	Sneut_tab=Sneut_tab+1
        S_tab=S_tab+1
        id_Sneut(Sneut_tab)=index
	Sid_Sneut(Sneut_tab)=S_tab
        Sneut_id(index)=Sneut_tab

	write(*,*)'reading table for analysis: e+ e- -> bbh -> 2b 2tau DELPHI (hep-ex/0410017)'

! information about how LEP data is stored in file:	 
	Mhmin(S_tab)=4D0
	Mhmax(S_tab)=50D0
	sep(S_tab)=1D0

	ntot=nint((Mhmax(S_tab)-Mhmin(S_tab))/sep(S_tab))+1
!	write(*,*)'ntot=',ntot
! ntot is expected to be 47. 

	open(10,file=INSTDIR//"/LEPtables/Delphi_yuk_h_bbtautau.txt")

! first 5 lines are not needed	 
        do i=1,5
	  read(10,*)  	 
        end do	 	
		 
        do i=1,ntot
	  read(10,*) Mh1dummy,S95_1d(2,S_tab,i),S95_1d(1,S_tab,i)
        enddo

        close(10)	 		
#endif


#ifdef OPALbbh_2b2tau
        index=index+1
        lep_publd(index)=.TRUE.
	nOPALbbh_2b2tau=index	

	Sneut_tab=Sneut_tab+1
        S_tab=S_tab+1
        id_Sneut(Sneut_tab)=index
	Sid_Sneut(Sneut_tab)=S_tab
        Sneut_id(index)=Sneut_tab

	write(*,*)'reading table for analysis: e+ e- -> bbh -> 2b 2tau OPAL (hep-ex/0111010)'

! information about how LEP data is stored in file:	 
	Mhmin(S_tab)=4D0
	Mhmax(S_tab)=12D0
	sep(S_tab)=1D0

	ntot=nint((Mhmax(S_tab)-Mhmin(S_tab))/sep(S_tab))+1

	open(10,file=INSTDIR//"/LEPtables/OPAL_yuk_h_bbtautau.txt")

! first 5 lines are not needed	 
        do i=1,5
	  read(10,*)  	 
        end do	 	
		 
        do i=1,ntot
	  read(10,*) Mh1dummy,S95_1d(2,S_tab,i),S95_1d(1,S_tab,i)
        enddo

        close(10)	 		
#endif


#ifdef DELPHIbba_2b2tau
        index=index+1
        lep_publd(index)=.TRUE.
	nDELPHIbba_2b2tau=index	

	Sneut_tab=Sneut_tab+1
        S_tab=S_tab+1
        id_Sneut(Sneut_tab)=index
	Sid_Sneut(Sneut_tab)=S_tab
        Sneut_id(index)=Sneut_tab

! new 8) channel e+ e- -> bba -> 2b2tau
	write(*,*)'reading table for analysis: e+ e- -> bba -> 2b 2tau DELPHI (hep-ex/0410017)'

! information about how LEP data is stored in file:	 
	Mhmin(S_tab)=4D0
	Mhmax(S_tab)=50D0
	sep(S_tab)=1D0

	ntot=nint((Mhmax(S_tab)-Mhmin(S_tab))/sep(S_tab))+1
! ntot is expected to be 47. 

	open(10,file=INSTDIR//"/LEPtables/Delphi_yuk_a_bbtautau.txt")

! first 5 lines are not needed	 
        do i=1,5
	  read(10,*)  	 
        end do	 	
		 
        do i=1,ntot
	  read(10,*) Mh1dummy,S95_1d(2,S_tab,i),S95_1d(1,S_tab,i)
        enddo

        close(10)	 		
#endif


#ifdef OPALbba_2b2tau
        index=index+1
        lep_publd(index)=.TRUE.
	nOPALbba_2b2tau=index	

	Sneut_tab=Sneut_tab+1
        S_tab=S_tab+1
        id_Sneut(Sneut_tab)=index
	Sid_Sneut(Sneut_tab)=S_tab
        Sneut_id(index)=Sneut_tab

	write(*,*)'reading table for analysis: e+ e- -> bba -> 2b 2tau OPAL (hep-ex/0111010)'

! information about how LEP data is stored in file:	 
	Mhmin(S_tab)=4D0
	Mhmax(S_tab)=12D0
	sep(S_tab)=1D0

	ntot=nint((Mhmax(S_tab)-Mhmin(S_tab))/sep(S_tab))+1

	open(10,file=INSTDIR//"/LEPtables/OPAL_yuk_a_bbtautau.txt")

! first 5 lines are not needed	 
        do i=1,5
	  read(10,*)  	 
        end do	 	
		 
        do i=1,ntot
	  read(10,*) Mh1dummy,S95_1d(2,S_tab,i),S95_1d(1,S_tab,i)
        enddo

        close(10)	 		
#endif




#ifdef DELPHItautauh_4tau
        index=index+1
        lep_publd(index)=.TRUE.
	nDELPHItautauh_4tau=index	

	Sneut_tab=Sneut_tab+1
        S_tab=S_tab+1
        id_Sneut(Sneut_tab)=index
	Sid_Sneut(Sneut_tab)=S_tab
        Sneut_id(index)=Sneut_tab

! new 9) channel e+ e- -> tau tau h -> 4 tau
	write(*,*)'reading table for analysis: e+ e- -> tau tau h -> 4 tau DELPHI (hep-ex/0410017)'

! information about how LEP data is stored in file:	 
	Mhmin(S_tab)=4D0
	Mhmax(S_tab)=27D0
	sep(S_tab)=1D0

	ntot=nint((Mhmax(S_tab)-Mhmin(S_tab))/sep(S_tab))+1
! ntot is expected to be 24. 

	open(10,file=INSTDIR//"/LEPtables/Delphi_yuk_h_tautautautau.txt")

! first 5 lines are not needed	 
        do i=1,5
	  read(10,*)  	 
        end do	 	
		 
        do i=1,ntot
	  read(10,*) Mh1dummy,S95_1d(2,S_tab,i),S95_1d(1,S_tab,i)
        enddo

        close(10)	 		
#endif


#ifdef DELPHItautaua_4tau
        index=index+1
        lep_publd(index)=.TRUE.
	nDELPHItautaua_4tau=index	

	Sneut_tab=Sneut_tab+1
        S_tab=S_tab+1
        id_Sneut(Sneut_tab)=index
	Sid_Sneut(Sneut_tab)=S_tab
        Sneut_id(index)=Sneut_tab

! new 10) channel e+ e- -> tau tau a -> 4 tau
	write(*,*)'reading table for analysis: e+ e- -> tau tau a -> 4 tau DELPHI (hep-ex/0410017)'

! information about how LEP data is stored in file:	 
	Mhmin(S_tab)=4D0
	Mhmax(S_tab)=26D0
	sep(S_tab)=1D0

	ntot=nint((Mhmax(S_tab)-Mhmin(S_tab))/sep(S_tab))+1
!	write(*,*)'ntot=',ntot
! ntot is expected to be 23. 

	open(10,file=INSTDIR//"/LEPtables/Delphi_yuk_a_tautautautau.txt")

! first 5 lines are not needed	 
        do i=1,5
	  read(10,*)  	 
        end do	 	
		 
        do i=1,ntot
	  read(10,*) Mh1dummy,S95_1d(2,S_tab,i),S95_1d(1,S_tab,i)
        enddo

        close(10)	 		
#endif


#ifdef DELPHIhZ_invisZ
        index=index+1
        lep_publd(index)=.TRUE.
	nDELPHIhZ_invisZ=index	

	Sneut_tab=Sneut_tab+1
        S_tab=S_tab+1
        id_Sneut(Sneut_tab)=index
	Sid_Sneut(Sneut_tab)=S_tab
        Sneut_id(index)=Sneut_tab

! channel e+ e- -> hZ, h -> invisible
	write(*,*)'reading table for analysis: e+ e- -> hZ, h -> invisible (DELPHI, hep-ex/0401022)'

! information about how LEP data is stored in file:	 
	Mhmin(S_tab)=40D0
	Mhmax(S_tab)=114D0
	sep(S_tab)=2D0

	ntot=nint((Mhmax(S_tab)-Mhmin(S_tab))/sep(S_tab))+1
! ntot is expected to be 38. 

	open(10,file=INSTDIR//"/LEPtables/Delphi_ee_hZ_invisZ.txt")

! first 5 lines are not needed	 
        do i=1,5
	  read(10,*)  	 
        end do	 	
		 
        do i=1,ntot
	  read(10,*) Mh1dummy,S95_1d(2,S_tab,i),S95_1d(1,S_tab,i)
        enddo

        close(10)	 		
#endif


#ifdef L3hZ_invisZ
        index=index+1
        lep_publd(index)=.TRUE.
	nL3hZ_invisZ=index	

	Sneut_tab=Sneut_tab+1
        S_tab=S_tab+1
        id_Sneut(Sneut_tab)=index
	Sid_Sneut(Sneut_tab)=S_tab
        Sneut_id(index)=Sneut_tab

! channel e+ e- -> hZ, h -> invisible
	write(*,*)'reading table for analysis: e+ e- -> hZ, h -> invisible (L3, hep-ex/0501033)'

! information about how LEP data is stored in file:	 
	Mhmin(S_tab)=50D0
	Mhmax(S_tab)=110D0
	sep(S_tab)=5D0

	ntot=nint((Mhmax(S_tab)-Mhmin(S_tab))/sep(S_tab))+1
! ntot is expected to be 13. 

	open(10,file=INSTDIR//"/LEPtables/L3_ee_hZ_invisZ.txt")

! first 5 lines are not needed	 
        do i=1,5
	  read(10,*)  	 
        end do	 	
		 
        do i=1,ntot
	  read(10,*) Mh1dummy,S95_1d(2,S_tab,i),S95_1d(1,S_tab,i)
        enddo

        close(10)	 		
#endif


#ifdef OPALhZ_invisZ
        index=index+1
        lep_publd(index)=.TRUE.
	nOPALhZ_invisZ=index	

	Sneut_tab=Sneut_tab+1
        S_tab=S_tab+1
        id_Sneut(Sneut_tab)=index
	Sid_Sneut(Sneut_tab)=S_tab
        Sneut_id(index)=Sneut_tab

! channel e+ e- -> hZ, h -> invisible
	write(*,*)'reading table for analysis: e+ e- -> hZ, h -> invisible (OPAL, hep-ex/0707.0373)'

! information about how LEP data is stored in file:	 
	Mhmin(S_tab)=5D0
	Mhmax(S_tab)=115D0
	sep(S_tab)=5D0

	ntot=nint((Mhmax(S_tab)-Mhmin(S_tab))/sep(S_tab))+1
! ntot is expected to be 23. 

	open(10,file=INSTDIR//"/LEPtables/OPAL_ee_hZ_invisZ.txt")

! first 5 lines are not needed	 
        do i=1,5
	  read(10,*)  	 
        end do	 	
		 
        do i=1,ntot
	  read(10,*) Mh1dummy,S95_1d(2,S_tab,i),S95_1d(1,S_tab,i)
        enddo

        close(10)	 		
#endif

	write(*,*)
        write(*,*)'single charged Higgs searches:'

#ifdef OPALhphm_2tau2nu
        index=index+1
        lep_publd(index)=.TRUE.
	nOPALhphm_2tau2nu=index	

	Schar_tab=Schar_tab+1
        S_tab=S_tab+1
	id_Schar(Schar_tab)=index
	Sid_Schar(Schar_tab)=S_tab
	Schar_id(index)=Schar_tab

! ch 1) channel e+ e- -> H+ H- -> 2 tau 2 nu
	write(*,*)'reading table for analysis: e+ e- ->  H+ H- -> 2 tau 2 nu (hep-ex/0812.0267)'

! information about how LEP data is stored in file:	 
	Mhmin(S_tab)=50D0
	Mhmax(S_tab)=93D0
	sep(S_tab)=1D0

	ntot=nint((Mhmax(S_tab)-Mhmin(S_tab))/sep(S_tab))+1
! ntot is expected to be 44. 

	open(10,file=INSTDIR//"/LEPtables/OPAL_HpHm_taunutaunu.txt")

! first 5 lines are not needed	 
        do i=1,5
	  read(10,*)  	 
        end do	 	
		 
        do i=1,ntot
	  read(10,*) Mh1dummy,S95_1d(2,S_tab,i),S95_1d(1,S_tab,i)
        enddo

        close(10)	 		
#endif


#ifdef OPALhphm_2jtaunu
        index=index+1
        lep_publd(index)=.TRUE.
	nOPALhphm_2jtaunu=index	

	Schar_tab=Schar_tab+1
        S_tab=S_tab+1
        id_Schar(Schar_tab)=index
	Sid_Schar(Schar_tab)=S_tab
        Schar_id(index)=Schar_tab

! ch 2) channel e+ e- -> H+ H- -> 2 jets tau nu
	write(*,*)'reading table for analysis: e+ e- ->  H+ H- -> 2 jets tau nu (hep-ex/0812.0267)'

! information about how LEP data is stored in file:	 
	Mhmin(S_tab)=50D0
	Mhmax(S_tab)=93D0
	sep(S_tab)=1D0

	ntot=nint((Mhmax(S_tab)-Mhmin(S_tab))/sep(S_tab))+1
!	write(*,*)'ntot=',ntot
! ntot is expected to be 44. 

	open(10,file=INSTDIR//"/LEPtables/OPAL_HpHm_qqtaunu.txt")

! first 5 lines are not needed	 
        do i=1,5
	  read(10,*)  	 
        end do	 	
		 
        do i=1,ntot
	  read(10,*) Mh1dummy,S95_1d(2,S_tab,i),S95_1d(1,S_tab,i)
        enddo

        close(10)	 		
#endif


#ifdef OPALhphm_4j
        index=index+1
        lep_publd(index)=.TRUE.
	nOPALhphm_4j=index	

	Schar_tab=Schar_tab+1
        S_tab=S_tab+1
        id_Schar(Schar_tab)=index
	Sid_Schar(Schar_tab)=S_tab
        Schar_id(index)=Schar_tab

! ch 3) channel e+ e- -> H+ H- -> 4 jets
	write(*,*)'reading table for analysis: e+ e- ->  H+ H- -> 4 jets (OPAL, hep-ex/0812.0267)'

! information about how LEP data is stored in file:	 
	Mhmin(S_tab)=50D0
	Mhmax(S_tab)=93D0
	sep(S_tab)=1D0

	ntot=nint((Mhmax(S_tab)-Mhmin(S_tab))/sep(S_tab))+1
!	write(*,*)'ntot=',ntot
! ntot is expected to be 44. 

	open(10,file=INSTDIR//"/LEPtables/OPAL_HpHm_qqqq.txt")

! first 5 lines are not needed	 
        do i=1,5
	  read(10,*)  	 
        end do	 	
		 
        do i=1,ntot
	  read(10,*) Mh1dummy,S95_1d(2,S_tab,i),S95_1d(1,S_tab,i)
        enddo

        close(10)	 		
#endif


#ifdef LEPhphm_4j
        index=index+1
        lep_publd(index)=.TRUE.
	nLEPhphm_4j=index	

	Schar_tab=Schar_tab+1
        S_tab=S_tab+1
        id_Schar(Schar_tab)=index
	Sid_Schar(Schar_tab)=S_tab
        Schar_id(index)=Schar_tab

! channel e+ e- -> H+ H- -> 4 jets
	write(*,*)'reading table for analysis: e+ e- ->  H+ H- -> 4 jets (LEP, hep-ex/0107031)'

! information about how LEP data is stored in file:	 
	Mhmin(S_tab)=60D0
	Mhmax(S_tab)=90D0
	sep(S_tab)=1D0

	ntot=nint((Mhmax(S_tab)-Mhmin(S_tab))/sep(S_tab))+1
!	write(*,*)'ntot=',ntot
! ntot is expected to be 31. 

	open(10,file=INSTDIR//"/LEPtables/LEP_HpHm_qqqq.txt")

! first 5 lines are not needed	 
        do i=1,5
	  read(10,*)  	 
        end do	 	
		 
        do i=1,ntot
	  read(10,*) Mh1dummy,S95_1d(2,S_tab,i),S95_1d(1,S_tab,i)
        enddo

        close(10)	 		
#endif


#ifdef DELPHIhphm_4j
        index=index+1
        lep_publd(index)=.TRUE.
	nDELPHIhphm_4j=index	

	Schar_tab=Schar_tab+1
        S_tab=S_tab+1
        id_Schar(Schar_tab)=index
	Sid_Schar(Schar_tab)=S_tab
        Schar_id(index)=Schar_tab

! channel e+ e- -> H+ H- -> 4 jets
	write(*,*)'reading table for analysis: e+ e- ->  H+ H- -> 4 jets (DELPHI, hep-ex/0404012)'

! information about how LEP data is stored in file:	 
	Mhmin(S_tab)=52D0
	Mhmax(S_tab)=94D0
	sep(S_tab)=2D0

	ntot=nint((Mhmax(S_tab)-Mhmin(S_tab))/sep(S_tab))+1
!	write(*,*)'ntot=',ntot
! ntot is expected to be 22. 

	open(10,file=INSTDIR//"/LEPtables/Delphi_HpHm_qqqq.txt")

! first 5 lines are not needed	 
        do i=1,5
	  read(10,*)  	 
        end do	 	
		 
        do i=1,ntot
	  read(10,*) Mh1dummy,S95_1d(2,S_tab,i),S95_1d(1,S_tab,i)
        enddo

        close(10)	 		
#endif


#ifdef DELPHIhphm_2tau2nu
        index=index+1
        lep_publd(index)=.TRUE.
	nDELPHIhphm_2tau2nu=index	

	Schar_tab=Schar_tab+1
        S_tab=S_tab+1
	id_Schar(Schar_tab)=index
	Sid_Schar(Schar_tab)=S_tab
	Schar_id(index)=Schar_tab

! channel e+ e- -> H+ H- -> 2 tau 2 nu
	write(*,*)'reading table for analysis: e+ e- ->  H+ H- -> 2 tau 2 nu (DELPHI, hep-ex/0404012)'

! information about how LEP data is stored in file:	 
	Mhmin(S_tab)=52D0
	Mhmax(S_tab)=94D0
	sep(S_tab)=2D0

	ntot=nint((Mhmax(S_tab)-Mhmin(S_tab))/sep(S_tab))+1
! ntot is expected to be 22.

	open(10,file=INSTDIR//"/LEPtables/Delphi_HpHm_taunutaunu.txt")

! first 5 lines are not needed	 
        do i=1,5
	  read(10,*)  	 
        end do	 	
		 
        do i=1,ntot
	  read(10,*) Mh1dummy,S95_1d(2,S_tab,i),S95_1d(1,S_tab,i)
        enddo

        close(10)	 		
#endif


	write(*,*)
        write(*,*)'double neutral Higgs searches:'

#ifdef LEPh2Z_h1h1Z_4bZ
        index=index+1
        lep_publd(index)=.TRUE.
	nLEPh2Z_h1h1Z_4bZ=index

	Dneut_tab=Dneut_tab+1
	id_Dneut(Dneut_tab)=index
	Dneut_id(index)=Dneut_tab

	write(*,*)'reading table 15 (EPJC 46(2006)547)'

! fill S95 from files
! row 0 and column 0 in LEP file contain higgs masses 
! and (0,0) ie top left set to -100	

	Mh1min(Dneut_tab)=1.0D0
        Mh1max(Dneut_tab)=60.0D0
        Mh2min(Dneut_tab)=2.0D0
        Mh2max(Dneut_tab)=120.0D0            

	sep1(Dneut_tab)=1.0D0
	sep2(Dneut_tab)=1.0D0
	
	imax=int((Mh2max(Dneut_tab)-Mh2min(Dneut_tab)))/sep2(Dneut_tab)+1
	jmax=int((Mh1max(Dneut_tab)-Mh1min(Dneut_tab)))/sep1(Dneut_tab)+1	
! (imax,jmax) is expected to be (119,60). 
	
	open(10,file=INSTDIR//"/LEPtables/table15full_pred.txt")
	open(11,file=INSTDIR//"/LEPtables/table15full_obs.txt")


	do i=0,imax  
         read(10,*) ( S95_2d(1,Dneut_tab,i,j), j=0,jmax )	   	 
         read(11,*) ( S95_2d(2,Dneut_tab,i,j), j=0,jmax )	   	 
        enddo
		
! set entries .ge. 100 to (-4): they will not be relevant
! Remark: If S95-value > 100 we don't apply the results anymore.
! In regions where LEP runs out of steam, S95 is sharply rising
! and there are usually a few more reasonable points with S95 > 100.
! Beyond that, S95 is abitrarily set to 10000.
! For S95 above about 100, interpolation is not very reliable anymore.
! Therefore, we stop using the tables there.
	do i=1,imax
	 do j=1,jmax
	  if(S95_2d(1,Dneut_tab,i,j) .ge. 100d0) S95_2d(1,Dneut_tab,i,j)= -4.0D0	
	  if(S95_2d(2,Dneut_tab,i,j)  .ge. 100d0) S95_2d(2,Dneut_tab,i,j) = -4.0D0	
	 enddo   	 
        enddo 	
	
	close(10)
        close(11)
#endif


#ifdef LEPh2Z_h1h1Z_4tauZ
        index=index+1
        lep_publd(index)=.TRUE.
	nLEPh2Z_h1h1Z_4tauZ=index

	Dneut_tab=Dneut_tab+1
        id_Dneut(Dneut_tab)=index
        Dneut_id(index)=Dneut_tab

	write(*,*)'reading table 16 (EPJC 46(2006)547)'

! fill S95 from files
! row 0 and column 0 in LEP file contain higgs masses 
! and (0,0) ie top left set to -100	

	Mh1min(Dneut_tab)=1.0D0
        Mh1max(Dneut_tab)=60.0D0
        Mh2min(Dneut_tab)=2.0D0
        Mh2max(Dneut_tab)=120.0D0            

	sep1(Dneut_tab)=1.0D0
	sep2(Dneut_tab)=1.0D0
	
	imax=int((Mh2max(Dneut_tab)-Mh2min(Dneut_tab)))/sep2(Dneut_tab)+1
	jmax=int((Mh1max(Dneut_tab)-Mh1min(Dneut_tab)))/sep1(Dneut_tab)+1	
! (imax,jmax) is expected to be (119,60). 
	
        open(10,file=INSTDIR//"/LEPtables/table16full_pred.txt")
        open(11,file=INSTDIR//"/LEPtables/table16full_obs.txt")

	do i=0,imax  
         read(10,*) ( S95_2d(1,Dneut_tab,i,j), j=0,jmax )	   	 
         read(11,*) ( S95_2d(2,Dneut_tab,i,j), j=0,jmax )	   	 
        enddo
		
! set entries .ge. 100 to (-4): they will not be relevant
! Remark: If S95-value > 100 we don't apply the results anymore.
! In regions where LEP runs out of steam, S95 is sharply rising
! and there are usually a few more reasonable points with S95 > 100.
! Beyond that, S95 is abitrarily set to 10000.
! For S95 above about 100, interpolation is not very reliable anymore.
! Therefore, we stop using the tables there.
	do i=1,imax
	 do j=1,jmax
	  if(S95_2d(1,Dneut_tab,i,j) .ge. 100d0) S95_2d(1,Dneut_tab,i,j)= -4.0D0	
	  if(S95_2d(2,Dneut_tab,i,j)  .ge. 100d0) S95_2d(2,Dneut_tab,i,j) = -4.0D0	
	 enddo   	 
        enddo 	
	
	close(10)
        close(11)
#endif




#ifdef LEPh2h1_4b
        index=index+1
        lep_publd(index)=.TRUE.
	nLEPh2h1_4b=index

	Dneut_tab=Dneut_tab+1
        id_Dneut(Dneut_tab)=index
        Dneut_id(index)=Dneut_tab

	write(*,*)'reading table 18 (EPJC 46(2006)547)'

	Mh1min(Dneut_tab)=1.0D0
        Mh1max(Dneut_tab)=180.0D0
        Mh2min(Dneut_tab)=1.0D0
        Mh2max(Dneut_tab)=180.0D0

	sep1(Dneut_tab)=1.0D0
	sep2(Dneut_tab)=1.0D0
	
	imax=int((Mh2max(Dneut_tab)-Mh2min(Dneut_tab)))/sep2(Dneut_tab)+1
	jmax=int((Mh1max(Dneut_tab)-Mh1min(Dneut_tab)))/sep1(Dneut_tab)+1	
! (imax,jmax) is expected to be (180,180). 

	open(10,file=INSTDIR//"/LEPtables/table18full_pred.txt")
	open(11,file=INSTDIR//"/LEPtables/table18full_obs.txt")

	do i=0,imax  
         read(10,*) ( S95_2d(1,Dneut_tab,i,j), j=0,jmax )	   	 
         read(11,*) ( S95_2d(2,Dneut_tab,i,j), j=0,jmax )	   	 
        enddo
		
! set entries .ge. 100 to (-4): they will not be relevant
! Remark: If S95-value > 100 we don't apply the results anymore.
! In regions where LEP runs out of steam, S95 is sharply rising
! and there are usually a few more reasonable points with S95 > 100.
! Beyond that, S95 is abitrarily set to 10000.
! For S95 above about 100, interpolation is not very reliable anymore.
! Therefore, we stop using the tables there.
	do i=1,imax
	 do j=1,jmax
	  if(S95_2d(1,Dneut_tab,i,j) .ge. 100d0) S95_2d(1,Dneut_tab,i,j)= -4.0D0	
	  if(S95_2d(2,Dneut_tab,i,j)  .ge. 100d0) S95_2d(2,Dneut_tab,i,j) = -4.0D0	
	 enddo   	 
        enddo 	
	
	close(10)
        close(11)
#endif


#ifdef LEPh2h1_4tau
        index=index+1
        lep_publd(index)=.TRUE.
	nLEPh2h1_4tau=index

	Dneut_tab=Dneut_tab+1
        id_Dneut(Dneut_tab)=index
        Dneut_id(index)=Dneut_tab

	write(*,*)'reading table 19 (EPJC 46(2006)547)'

	Mh1min(Dneut_tab)=1.0D0
        Mh1max(Dneut_tab)=180.0D0
        Mh2min(Dneut_tab)=1.0D0
        Mh2max(Dneut_tab)=180.0D0

	sep1(Dneut_tab)=1.0D0
	sep2(Dneut_tab)=1.0D0
	
	imax=int((Mh2max(Dneut_tab)-Mh2min(Dneut_tab)))/sep2(Dneut_tab)+1
	jmax=int((Mh1max(Dneut_tab)-Mh1min(Dneut_tab)))/sep1(Dneut_tab)+1	
! (imax,jmax) is expected to be (180,180). 

        open(10,file=INSTDIR//"/LEPtables/table19full_pred.txt")
        open(11,file=INSTDIR//"/LEPtables/table19full_obs.txt")

	do i=0,imax  
         read(10,*) ( S95_2d(1,Dneut_tab,i,j), j=0,jmax )	   	 
         read(11,*) ( S95_2d(2,Dneut_tab,i,j), j=0,jmax )	   	 
        enddo
		
! set entries .ge. 100 to (-4): they will not be relevant
! Remark: If S95-value > 100 we don't apply the results anymore.
! In regions where LEP runs out of steam, S95 is sharply rising
! and there are usually a few more reasonable points with S95 > 100.
! Beyond that, S95 is abitrarily set to 10000.
! For S95 above about 100, interpolation is not very reliable anymore.
! Therefore, we stop using the tables there.
	do i=1,imax
	 do j=1,jmax
	  if(S95_2d(1,Dneut_tab,i,j) .ge. 100d0) S95_2d(1,Dneut_tab,i,j)= -4.0D0	
	  if(S95_2d(2,Dneut_tab,i,j)  .ge. 100d0) S95_2d(2,Dneut_tab,i,j) = -4.0D0	
	 enddo   	 
        enddo 	
	
	close(10)
        close(11)
#endif




#ifdef LEPh2h1_h1h1h1_6b
        index=index+1
        lep_publd(index)=.TRUE.
	nLEPh2h1_h1h1h1_6b=index

	Dneut_tab=Dneut_tab+1
        id_Dneut(Dneut_tab)=index
        Dneut_id(index)=Dneut_tab

	write(*,*)'reading table 20 (EPJC 46(2006)547)'

	Mh1min(Dneut_tab)=1.0D0
        Mh1max(Dneut_tab)=90.0D0
        Mh2min(Dneut_tab)=2.0D0
        Mh2max(Dneut_tab)=180.0D0

	sep1(Dneut_tab)=1.0D0
	sep2(Dneut_tab)=1.0D0
	
	imax=int((Mh2max(Dneut_tab)-Mh2min(Dneut_tab)))/sep2(Dneut_tab)+1
	jmax=int((Mh1max(Dneut_tab)-Mh1min(Dneut_tab)))/sep1(Dneut_tab)+1	
! (imax,jmax) is expected to be (179,90). 

	open(10,file=INSTDIR//"/LEPtables/table20full_pred.txt")
	open(11,file=INSTDIR//"/LEPtables/table20full_obs.txt")

	do i=0,imax  
         read(10,*) ( S95_2d(1,Dneut_tab,i,j), j=0,jmax )	   	 
         read(11,*) ( S95_2d(2,Dneut_tab,i,j), j=0,jmax )	   	 
        enddo
		
! set entries .ge. 100 to (-4): they will not be relevant
! Remark: If S95-value > 100 we don't apply the results anymore.
! In regions where LEP runs out of steam, S95 is sharply rising
! and there are usually a few more reasonable points with S95 > 100.
! Beyond that, S95 is abitrarily set to 10000.
! For S95 above about 100, interpolation is not very reliable anymore.
! Therefore, we stop using the tables there.
	do i=1,imax
	 do j=1,jmax
	  if(S95_2d(1,Dneut_tab,i,j) .ge. 100d0) S95_2d(1,Dneut_tab,i,j)= -4.0D0	
	  if(S95_2d(2,Dneut_tab,i,j)  .ge. 100d0) S95_2d(2,Dneut_tab,i,j) = -4.0D0	
	 enddo   	 
        enddo 	
	
	close(10)
        close(11)
#endif


#ifdef LEPh2h1_h1h1h1_6tau
        index=index+1
        lep_publd(index)=.TRUE.
	nLEPh2h1_h1h1h1_6tau=index

	Dneut_tab=Dneut_tab+1
        id_Dneut(Dneut_tab)=index
        Dneut_id(index)=Dneut_tab

	write(*,*)'reading table 21 (EPJC 46(2006)547)'

	Mh1min(Dneut_tab)=1.0D0
        Mh1max(Dneut_tab)=90.0D0
        Mh2min(Dneut_tab)=2.0D0
        Mh2max(Dneut_tab)=180.0D0

	sep1(Dneut_tab)=1.0D0
	sep2(Dneut_tab)=1.0D0
	
	imax=int((Mh2max(Dneut_tab)-Mh2min(Dneut_tab)))/sep2(Dneut_tab)+1
	jmax=int((Mh1max(Dneut_tab)-Mh1min(Dneut_tab)))/sep1(Dneut_tab)+1	
! (imax,jmax) is expected to be (179,90). 

        open(10,file=INSTDIR//"/LEPtables/table21full_pred.txt")
        open(11,file=INSTDIR//"/LEPtables/table21full_obs.txt")

	do i=0,imax  
         read(10,*) ( S95_2d(1,Dneut_tab,i,j), j=0,jmax )	   	 
         read(11,*) ( S95_2d(2,Dneut_tab,i,j), j=0,jmax )	   	 
        enddo
		
! set entries .ge. 100 to (-4): they will not be relevant
! Remark: If S95-value > 100 we don't apply the results anymore.
! In regions where LEP runs out of steam, S95 is sharply rising
! and there are usually a few more reasonable points with S95 > 100.
! Beyond that, S95 is abitrarily set to 10000.
! For S95 above about 100, interpolation is not very reliable anymore.
! Therefore, we stop using the tables there.
	do i=1,imax
	 do j=1,jmax
	  if(S95_2d(1,Dneut_tab,i,j) .ge. 100d0) S95_2d(1,Dneut_tab,i,j)= -4.0D0	
	  if(S95_2d(2,Dneut_tab,i,j)  .ge. 100d0) S95_2d(2,Dneut_tab,i,j) = -4.0D0	
	 enddo   	 
        enddo 	
	
	close(10)
        close(11)
#endif


#ifdef LEPh2Z_h1h1Z_2b2tau
        index=index+1
        lep_publd(index)=.TRUE.
	nLEPh2Z_h1h1Z_2b2tau=index

	Dneut_tab=Dneut_tab+1
        id_Dneut(Dneut_tab)=index
        Dneut_id(index)=Dneut_tab

	write(*,*)'reading table 22 (EPJC 46(2006)547)'

! fill S95 from files
! row 0 and column 0 in LEP file contain higgs masses 
! and (0,0) ie top left set to -100	

	Mh1min(Dneut_tab)=1.0D0
        Mh1max(Dneut_tab)=60.0D0
        Mh2min(Dneut_tab)=2.0D0
        Mh2max(Dneut_tab)=120.0D0            

	sep1(Dneut_tab)=1.0D0
	sep2(Dneut_tab)=1.0D0
	
	imax=int((Mh2max(Dneut_tab)-Mh2min(Dneut_tab)))/sep2(Dneut_tab)+1
	jmax=int((Mh1max(Dneut_tab)-Mh1min(Dneut_tab)))/sep1(Dneut_tab)+1	
! (imax,jmax) is expected to be (119,60). 
	
        open(10,file=INSTDIR//"/LEPtables/table22full_pred.txt")
        open(11,file=INSTDIR//"/LEPtables/table22full_obs.txt")

	do i=0,imax  
         read(10,*) ( S95_2d(1,Dneut_tab,i,j), j=0,jmax )	   	 
         read(11,*) ( S95_2d(2,Dneut_tab,i,j), j=0,jmax )	   	 
        enddo
		
! set entries .ge. 100 to (-4): they will not be relevant
! Remark: If S95-value > 100 we don't apply the results anymore.
! In regions where LEP runs out of steam, S95 is sharply rising
! and there are usually a few more reasonable points with S95 > 100.
! Beyond that, S95 is abitrarily set to 10000.
! For S95 above about 100, interpolation is not very reliable anymore.
! Therefore, we stop using the tables there.
	do i=1,imax
	 do j=1,jmax
	  if(S95_2d(1,Dneut_tab,i,j) .ge. 100d0) S95_2d(1,Dneut_tab,i,j)= -4.0D0	
	  if(S95_2d(2,Dneut_tab,i,j)  .ge. 100d0) S95_2d(2,Dneut_tab,i,j) = -4.0D0	
	 enddo   	 
        enddo 	
	
	close(10)
        close(11)
#endif


#ifdef LEPh2h1_2b2tau
        index=index+1
        lep_publd(index)=.TRUE.
	nLEPh2h1_2b2tau=index

	Dneut_tab=Dneut_tab+1
        id_Dneut(Dneut_tab)=index
        Dneut_id(index)=Dneut_tab

	write(*,*)'reading table 23 (EPJC 46(2006)547)'

	Mh1min(Dneut_tab)=1.0D0
        Mh1max(Dneut_tab)=180.0D0
        Mh2min(Dneut_tab)=1.0D0
        Mh2max(Dneut_tab)=180.0D0

	sep1(Dneut_tab)=1.0D0
	sep2(Dneut_tab)=1.0D0
	
	imax=int((Mh2max(Dneut_tab)-Mh2min(Dneut_tab)))/sep2(Dneut_tab)+1
	jmax=int((Mh1max(Dneut_tab)-Mh1min(Dneut_tab)))/sep1(Dneut_tab)+1	
! (imax,jmax) is expected to be (180,180). 

        open(10,file=INSTDIR//"/LEPtables/table23full_pred.txt")
        open(11,file=INSTDIR//"/LEPtables/table23full_obs.txt")

	do i=0,imax  
         read(10,*) ( S95_2d(1,Dneut_tab,i,j), j=0,jmax )	   	 
         read(11,*) ( S95_2d(2,Dneut_tab,i,j), j=0,jmax )	   	 
        enddo
		
! set entries .ge. 100 to (-4): they will not be relevant
! Remark: If S95-value > 100 we don't apply the results anymore.
! In regions where LEP runs out of steam, S95 is sharply rising
! and there are usually a few more reasonable points with S95 > 100.
! Beyond that, S95 is abitrarily set to 10000.
! For S95 above about 100, interpolation is not very reliable anymore.
! Therefore, we stop using the tables there.
	do i=1,imax
	 do j=1,jmax
	  if(S95_2d(1,Dneut_tab,i,j) .ge. 100d0) S95_2d(1,Dneut_tab,i,j)= -4.0D0	
	  if(S95_2d(2,Dneut_tab,i,j)  .ge. 100d0) S95_2d(2,Dneut_tab,i,j) = -4.0D0	
	 enddo   	 
        enddo 	
	
	close(10)
        close(11)
#endif


#ifdef LEPh2h1_2tau2b
        index=index+1
        lep_publd(index)=.TRUE.
	nLEPh2h1_2tau2b=index

	Dneut_tab=Dneut_tab+1
        id_Dneut(Dneut_tab)=index
        Dneut_id(index)=Dneut_tab

	write(*,*)'reading table 24 (EPJC 46(2006)547)'

	Mh1min(Dneut_tab)=1.0D0
        Mh1max(Dneut_tab)=180.0D0
        Mh2min(Dneut_tab)=1.0D0
        Mh2max(Dneut_tab)=180.0D0

	sep1(Dneut_tab)=1.0D0
	sep2(Dneut_tab)=1.0D0
	
	imax=int((Mh2max(Dneut_tab)-Mh2min(Dneut_tab)))/sep2(Dneut_tab)+1
	jmax=int((Mh1max(Dneut_tab)-Mh1min(Dneut_tab)))/sep1(Dneut_tab)+1	
! (imax,jmax) is expected to be (180,180). 

        open(10,file=INSTDIR//"/LEPtables/table24full_pred.txt")
        open(11,file=INSTDIR//"/LEPtables/table24full_obs.txt")

	do i=0,imax  
         read(10,*) ( S95_2d(1,Dneut_tab,i,j), j=0,jmax )	   	 
         read(11,*) ( S95_2d(2,Dneut_tab,i,j), j=0,jmax )	   	 
        enddo

! set entries .ge. 100 to (-4): they will not be relevant
! Remark: If S95-value > 100 we don't apply the results anymore.
! In regions where LEP runs out of steam, S95 is sharply rising
! and there are usually a few more reasonable points with S95 > 100.
! Beyond that, S95 is abitrarily set to 10000.
! For S95 above about 100, interpolation is not very reliable anymore.
! Therefore, we stop using the tables there.
	do i=1,imax
	 do j=1,jmax
	  if(S95_2d(1,Dneut_tab,i,j) .ge. 100d0) S95_2d(1,Dneut_tab,i,j)= -4.0D0	
	  if(S95_2d(2,Dneut_tab,i,j)  .ge. 100d0) S95_2d(2,Dneut_tab,i,j) = -4.0D0	
	 enddo   	 
        enddo 	
	
	close(10)
        close(11)
#endif

	write(*,*)'finished reading LEP tables'
	write(*,*)


	nLEP_NSneutCHN=Sneut_tab
	nLEP_NScharCHN=Schar_tab
	nLEP_NSCHN=nLEP_NSneutCHN+nLEP_NScharCHN

c>>check
	if(S_tab .ne. nLEP_NSCHN) then
	  write(*,*)'Something is going wrong!!'
	  stop
	endif
c<<
	nLEP_NDCHN=Dneut_tab
c>> currently not used
c	number_lep_analyses=nLEP_NSCHN+nLEP_NDCHN
c<<

c>> check
c	write(*,*)'nLEP_NSneutCHN=',nLEP_NSneutCHN
c        write(*,*)'nLEP_NScharCHN=',nLEP_NScharCHN
c        write(*,*)'nLEP_NSCHN=',nLEP_NSCHN
c        write(*,*)'nLEP_NDCHN=',nLEP_NDCHN
c<<
	write(*,*)'Number of LEP analyses :'
	write(*,*)'   single neutral Higgs analyses :',nLEP_NSneutCHN
	write(*,*)'   single charged Higgs analyses :',nLEP_NScharCHN
	write(*,*)'   double neutral Higgs analyses :',nLEP_NDCHN
	write(*,*)'                        total    :',nLEP_NSneutCHN+nLEP_NScharCHN+nLEP_NDCHN
	write(*,*)

c>> currently not used
c        write(*,*)'number_lep_analyses=',number_lep_analyses
c<<

	end




!***********************************************************
	subroutine check_LEP_bound(allowed,result_flag,
     &		mlratio,obsratio,most_sens_predratio,num_H_combined,
     &		final_h_comb,model_R_sig_BR)
!***********************************************************
! Output:
!	allowed : logical variable, TRUE if scenario is allowed at
!		95% confidence level (This is determined, using the procedure
!	 	described by Philipp Bechtle.), FALSE otherwise.
!	result_flag
!		1 if point is allowed, 
!		0 if point is not allowed
!	       -1 otherwise (theoretically inaccessible, etc.)
!	mlratio : gives information about which table was used for the 
!		experimental comparison.
!	obsratio, most_sens_predratio, num_H_combined, final_h_comb, model_R_sig_BR 
!--------------------------------
	implicit none 
#include "input_cb.h"
	logical allowed,testKR,useTEV,useLEP,useSingH,predicted_limit

! This form assumes that there only single Higgs channel analyses
! need a applicability check:
        logical table_applicable(LEP_NSCHN*NHMAX)

	integer my_maxloc,output_chan,index,baseindex,offset,chann,runid
	integer result_flag,num_H_combined
	integer i,j,k,n,m,mlratio,mlindex,nhelp
	integer mlratiotemp(1)	
	double precision R_sig_BR(LEP_NSCHN*NHMAX+LEP_NDCHN*NHMAX**2)
        double precision R_sig_BR_sing(LEP_NSCHN*NHMAX+LEP_NDCHN*NHMAX**2)
        double precision predratio(LEP_NSCHN*NHMAX+LEP_NDCHN*NHMAX**2)
	double precision obsratio,most_sens_predratio

	logical higgs_combination(LEP_NSCHN*NHMAX+LEP_NDCHN*NHMAX**2,NHMAX),final_h_comb(NHMAX)
	integer sim_mass_sum(LEP_NSCHN*NHMAX+LEP_NDCHN*NHMAX**2)
        double precision Mh_avg(LEP_NSCHN*NHMAX),delta_Mh,delta_Mhp
	double precision log2dble,model_R_sig_BR
	double precision BRhjjj(NHMAX)

	integer findi,findj,LEP_reaction_number,LEP_analysis_number,number_LEP_channels
#include "tables-indexing-lep_cb.h"

	delta_Mh = delta_Mh_LEP
c deactivated	delta_Mhp = delta_Mhplus_LEP
	delta_Mhp = 0d0

        call determine_topology_flags(whichexpt,
     &          useTEV,useLEP,useSingH)



! tlist(n) will tell the subroutines check_LEP_bound1 and check_LEP_bound2
! which LEP tables to look at
	do m=0,nLEP_NSneutCHN-1,1
	do n=1,NHIGGS
	 R_sig_BR_sing(n+NHIGGS*m)=0d0
         R_sig_BR(n+NHIGGS*m)=0d0
	 sim_mass_sum(n+NHIGGS*m)=0
	 do i=1,NHIGGS
          higgs_combination(n+NHIGGS*m,i)=.FALSE.
	 enddo
	enddo
	enddo

	offset = nLEP_NSneutCHN*NHIGGS
	do m=0,nLEP_NScharCHN-1,1
	do n=1,NHPLUS
	 R_sig_BR_sing(offset+n+NHPLUS*m)=0d0
         R_sig_BR(offset+n+NHPLUS*m)=0d0
	 sim_mass_sum(offset+n+NHPLUS*m)=0
	 do i=1,NHPLUS
          higgs_combination(offset+n+NHPLUS*m,i)=.FALSE.
	 enddo
	enddo
	enddo

        offset = nLEP_NSneutCHN*NHIGGS+nLEP_NScharCHN*NHPLUS
	do m=0,nLEP_NDCHN-1,1
	do n=1,NHIGGS**2
	 R_sig_BR_sing(offset+n+m*NHIGGS**2)=0d0
	 R_sig_BR(offset+n+m*NHIGGS**2)=0d0
	 do i=1,NHIGGS
	  higgs_combination(offset+n+m*NHIGGS**2,i)=.FALSE.
         enddo
	enddo				
	enddo

	do n=1,NHIGGS
cnot needed yet	    if( (Mh(n) .ge. Mhmin_SMcalc) .and. (Mh(n) .le. Mhmax_SMcalc) ) then 
	      do m=0,nLEP_NSneutCHN-1,1
	       call check_LEP_applicability(table_applicable(n+NHIGGS*m), m+1, n)
	      enddo
cnot needed yet	    endif 
	enddo
	do n=1,NHPLUS
cnot needed yet	    if( (Mh(n) .ge. Mhmin_SMcalc) .and. (Mh(n) .le. Mhmax_SMcalc) ) then 
	      do m=0,nLEP_NScharCHN-1,1
	       call check_LEP_applicability(
     &			table_applicable(NHIGGS*nLEP_NSneutCHN+n+NHPLUS*m), nLEP_NSneutCHN+m+1, n)
	      enddo
cnot needed yet	    endif 
	enddo




! a0) calc single sigma x BR values (no combinations of near mass Higgs bosons)

!    single Higgs analyses

	do n=1,NHIGGS
	  BRhjjj(n)=BR_hjss(n)+BR_hjcc(n)+BR_hjbb(n)+BR_hjgg(n)
#ifdef LEPhZ_bbZ
	if((nLEPhZ_bbZ) .gt. 0) then
	  R_sig_BR_sing(n+NHIGGS*(Sneut_id(nLEPhZ_bbZ) -1))=CS_lep_hjZ_ratio(n)*BR_hjbb(n)
	endif
#endif
#ifdef LEPhZ_tautauZ
        if((nLEPhZ_tautauZ) .gt. 0) then
          R_sig_BR_sing(n+NHIGGS*(Sneut_id(nLEPhZ_tautauZ) -1))=CS_lep_hjZ_ratio(n)*BR_hjtautau(n)	
        endif
#endif
#ifdef LEPhZ_anyZ
        if((nLEPhZ_anyZ) .gt. 0) then
          R_sig_BR_sing(n+NHIGGS*(Sneut_id(nLEPhZ_anyZ) -1))=CS_lep_hjZ_ratio(n)
        endif
#endif
#ifdef LEPhZ_gamgamZ	  
        if((nLEPhZ_gamgamZ) .gt. 0) then
	  R_sig_BR_sing(n+NHIGGS*(Sneut_id(nLEPhZ_gamgamZ) -1))=CS_lep_hjZ_ratio(n)*BR_hjgaga(n)
        endif
#endif
#ifdef LEPhZ_jjZ  
        if((nLEPhZ_jjZ) .gt. 0) then
	  R_sig_BR_sing(n+NHIGGS*(Sneut_id(nLEPhZ_jjZ) -1))=CS_lep_hjZ_ratio(n)*BRhjjj(n)
        endif
#endif
#ifdef LEPhZ_invisZ
        if((nLEPhZ_invisZ) .gt. 0) then
          R_sig_BR_sing(n+NHIGGS*(Sneut_id(nLEPhZ_invisZ) -1))=CS_lep_hjZ_ratio(n)*BR_hjinvisible(n)
        endif
#endif
#ifdef DELPHIhZ_invisZ
        if((nDELPHIhZ_invisZ) .gt. 0) then
          R_sig_BR_sing(n+NHIGGS*(Sneut_id(nDELPHIhZ_invisZ) -1))=CS_lep_hjZ_ratio(n)*BR_hjinvisible(n)
        endif
#endif
#ifdef L3hZ_invisZ
        if((nL3hZ_invisZ) .gt. 0) then
          R_sig_BR_sing(n+NHIGGS*(Sneut_id(nL3hZ_invisZ) -1))=CS_lep_hjZ_ratio(n)*BR_hjinvisible(n)
        endif
#endif
#ifdef OPALhZ_invisZ
        if((nOPALhZ_invisZ) .gt. 0) then
          R_sig_BR_sing(n+NHIGGS*(Sneut_id(nOPALhZ_invisZ) -1))=CS_lep_hjZ_ratio(n)*BR_hjinvisible(n)
        endif
#endif
#ifdef DELPHIbbh_4b
        if((nDELPHIbbh_4b) .gt. 0) then
 	  R_sig_BR_sing(n+NHIGGS*(Sneut_id(nDELPHIbbh_4b) -1))=
     &		log2dble(table_applicable(n+NHIGGS*(Sneut_id(nDELPHIbbh_4b) -1)))
     &		*CS_lep_bbhj_ratio(n)*BR_hjbb(n)
        endif
#endif
#ifdef DELPHIbba_4b
        if((nDELPHIbba_4b) .gt. 0) then
 	  R_sig_BR_sing(n+NHIGGS*(Sneut_id(nDELPHIbba_4b) -1))=
     &		log2dble(table_applicable(n+NHIGGS*(Sneut_id(nDELPHIbba_4b) -1)))
     &		*CS_lep_bbhj_ratio(n)*BR_hjbb(n)
        endif
#endif
#ifdef DELPHIbbh_2b2tau
! According to hep-ex/0410017 e+e- ->Z -> b b h/a has much larger CS than
! e+e- ->Z -> tau tau h/a due to much stronger bbZ-coupling compared to the
! tautauZ-coupling. Thus, the contribution of e+e- ->Z -> tau tau h/a
! is not considered.
        if((nDELPHIbbh_2b2tau) .gt. 0) then
 	  R_sig_BR_sing(n+NHIGGS*(Sneut_id(nDELPHIbbh_2b2tau) -1))=
     &		log2dble(table_applicable(n+NHIGGS*(Sneut_id(nDELPHIbbh_2b2tau) -1)))
     &		*CS_lep_bbhj_ratio(n)*BR_hjtautau(n)
        endif
#endif
#ifdef OPALbbh_2b2tau
! According to hep-ex/0410017 e+e- ->Z -> b b h/a has much larger CS than
! e+e- ->Z -> tau tau h/a due to much stronger bbZ-coupling compared to the
! tautauZ-coupling. Thus, the contribution of e+e- ->Z -> tau tau h/a
! is not considered.
        if((nOPALbbh_2b2tau) .gt. 0) then
 	  R_sig_BR_sing(n+NHIGGS*(Sneut_id(nOPALbbh_2b2tau) -1))=
     &		log2dble(table_applicable(n+NHIGGS*(Sneut_id(nOPALbbh_2b2tau) -1)))
     &		*CS_lep_bbhj_ratio(n)*BR_hjtautau(n)
        endif
#endif
#ifdef DELPHIbba_2b2tau
! According to hep-ex/0410017 e+e- ->Z -> b b h/a has much larger CS than
! e+e- ->Z -> tau tau h/a due to much stronger bbZ-coupling compared to the
! tautauZ-coupling. Thus, the contribution of e+e- ->Z -> tau tau h/a
! is not considered.
        if((nDELPHIbba_2b2tau) .gt. 0) then
 	  R_sig_BR_sing(n+NHIGGS*(Sneut_id(nDELPHIbba_2b2tau) -1))=
     &		log2dble(table_applicable(n+NHIGGS*(Sneut_id(nDELPHIbba_2b2tau) -1)))
     &		*CS_lep_bbhj_ratio(n)*BR_hjtautau(n)
        endif
#endif
#ifdef OPALbba_2b2tau
! According to hep-ex/0410017 e+e- ->Z -> b b h/a has much larger CS than
! e+e- ->Z -> tau tau h/a due to much stronger bbZ-coupling compared to the
! tautauZ-coupling. Thus, the contribution of e+e- ->Z -> tau tau h/a
! is not considered.
        if((nOPALbba_2b2tau) .gt. 0) then
 	  R_sig_BR_sing(n+NHIGGS*(Sneut_id(nOPALbba_2b2tau) -1))=
     &		log2dble(table_applicable(n+NHIGGS*(Sneut_id(nOPALbba_2b2tau) -1)))
     &		*CS_lep_bbhj_ratio(n)*BR_hjtautau(n)
        endif
#endif


#ifdef DELPHItautauh_4tau
        if((nDELPHItautauh_4tau) .gt. 0) then
 	  R_sig_BR_sing(n+NHIGGS*(Sneut_id(nDELPHItautauh_4tau) -1))=
     &		log2dble(table_applicable(n+NHIGGS*(Sneut_id(nDELPHItautauh_4tau) -1)))
     &		*CS_lep_tautauhj_ratio(n)*BR_hjtautau(n)
        endif
#endif
#ifdef DELPHItautaua_4tau
        if((nDELPHItautaua_4tau) .gt. 0) then
 	  R_sig_BR_sing(n+NHIGGS*(Sneut_id(nDELPHItautaua_4tau) -1))=
     &		log2dble(table_applicable(n+NHIGGS*(Sneut_id(nDELPHItautaua_4tau) -1)))
     &		*CS_lep_tautauhj_ratio(n)*BR_hjtautau(n)
        endif
#endif
	enddo


!     charged Higgs analyses
	offset = nLEP_NSneutCHN*NHIGGS

	do n=1,NHPLUS
#ifdef OPALhphm_2tau2nu
        if((nOPALhphm_2tau2nu) .gt. 0) then
	  chann=offset+n+NHPLUS*(Schar_id(nOPALhphm_2tau2nu) -1)
	  R_sig_BR_sing(chann)=CS_lep_HpjHmj_ratio(n)*(BR_Hpjtaunu(n))**2
        endif
#endif
#ifdef DELPHIhphm_2tau2nu
        if((nDELPHIhphm_2tau2nu) .gt. 0) then
	  chann=offset+n+NHPLUS*(Schar_id(nDELPHIhphm_2tau2nu) -1)
	  R_sig_BR_sing(chann)=CS_lep_HpjHmj_ratio(n)*(BR_Hpjtaunu(n))**2
        endif
#endif
#ifdef OPALhphm_2jtaunu
        if((nOPALhphm_2jtaunu) .gt. 0) then
	  chann=offset+n+NHPLUS*(Schar_id(nOPALhphm_2jtaunu) -1)
	  R_sig_BR_sing(chann)=CS_lep_HpjHmj_ratio(n)
     &		*2d0*( BR_Hpjcs(n) + BR_Hpjcb(n) )*BR_Hpjtaunu(n)
        endif
#endif
#ifdef OPALhphm_4j
        if((nOPALhphm_4j) .gt. 0) then
	  chann=offset+n+NHPLUS*(Schar_id(nOPALhphm_4j) -1)
	  R_sig_BR_sing(chann)=CS_lep_HpjHmj_ratio(n)
     &		*( BR_Hpjcs(n) + BR_Hpjcb(n) )**2
        endif
#endif
#ifdef LEPhphm_4j
        if((nLEPhphm_4j) .gt. 0) then
	  chann=offset+n+NHPLUS*(Schar_id(nLEPhphm_4j) -1)
	  R_sig_BR_sing(chann)=CS_lep_HpjHmj_ratio(n)
     &		*( BR_Hpjcs(n) + BR_Hpjcb(n) )**2
        endif
#endif
#ifdef DELPHIhphm_4j
        if((nDELPHIhphm_4j) .gt. 0) then
	  chann=offset+n+NHPLUS*(Schar_id(nDELPHIhphm_4j) -1)
	  R_sig_BR_sing(chann)=CS_lep_HpjHmj_ratio(n)
     &		*( BR_Hpjcs(n) + BR_Hpjcb(n) )**2
        endif
#endif
	enddo

!    double Higgs channels: not needed because no adding of signals 
!    is (so far) done for double Higgs channels


c>> check
c          offset = nLEP_NSneutCHN*NHIGGS
c          do n=1,NHPLUS
c		do baseindex=offset,offset+NHPLUS*(nLEP_NScharCHN-1),NHPLUS
c	  	write(*,*)'n=',n
c	  	write(*,*)'R_sig_BR_sing(',n+baseindex,')=',R_sig_BR_sing(n+baseindex)
c		enddo
c	  enddo
c<<


! a) calc sum of sigma x BR contributions to R_sig_BR

!   single neutral Higgs analyses

!   R_sig_BR(n) will be compared to the S95 values in the LEP tables

	  do n=1,NHIGGS
	    do i=1,NHIGGS
	      if( (Mh(i) .ge. Mh(n)) .and. ( (Mh(i)-Mh(n)) .le. delta_Mh) ) then
                do baseindex=0,NHIGGS*(nLEP_NSneutCHN-1),NHIGGS
ccheck	write(*,*) 'baseindex=',baseindex
		  if(R_sig_BR_sing(i+baseindex)*R_sig_BR_sing(n+baseindex) .ne. 0d0) then
	            R_sig_BR(n+baseindex)=R_sig_BR(n+baseindex)+R_sig_BR_sing(i+baseindex)
		  endif
		enddo
	      endif
	    enddo
	  enddo


!   single charged Higgs analyses
            baseindex=NHPLUS*m+nLEP_NSneutCHN*NHIGGS

	  offset = nLEP_NSneutCHN*NHIGGS
	  do n=1,NHPLUS
	    do i=1,NHPLUS
	      if( (MHplus(i) .ge. MHplus(n)) .and. ( (MHplus(i)-MHplus(n)) .le. delta_Mhp) ) then
ccheck	write(*,*) 'baseindex=',baseindex
                do baseindex=offset,offset+NHPLUS*(nLEP_NScharCHN-1),NHPLUS
		  if(R_sig_BR_sing(i+baseindex)*R_sig_BR_sing(n+baseindex) .ne. 0d0) then
	            R_sig_BR(n+baseindex)=R_sig_BR(n+baseindex)+R_sig_BR_sing(i+baseindex)
		  endif
		enddo
	      endif
	    enddo
	  enddo

c>> check
c          offset = nLEP_NSneutCHN*NHIGGS
c          do n=1,NHPLUS
c		do baseindex=offset,offset+NHPLUS*(nLEP_NScharCHN-1),NHPLUS
c	  	write(*,*)'n=',n
c	  	write(*,*)'R_sig_BR_sing(',n+baseindex,')=',R_sig_BR_sing(n+baseindex)
c		enddo
c	  enddo
c<<


!   double Higgs channels

	  if(.not. useSingH) then	  

	  offset=nLEP_NSneutCHN*NHIGGS+nLEP_NScharCHN*NHPLUS

	  do n=1,NHIGGS**2
	   i=findi(NHIGGS,n)
	   j=findj(NHIGGS,n)
#ifdef LEPh2Z_h1h1Z_4bZ
	   chann=n+(Dneut_id(nLEPh2Z_h1h1Z_4bZ) -1)*NHIGGS**2+offset
	   R_sig_BR(chann)=CS_lep_hjZ_ratio(j)*BR_hjhihi(j,i)*BR_hjbb(i)**2.0D0	
#endif
#ifdef LEPh2Z_h1h1Z_4tauZ
	   chann=n+(Dneut_id(nLEPh2Z_h1h1Z_4tauZ) -1)*NHIGGS**2+offset
	   R_sig_BR(chann)=CS_lep_hjZ_ratio(j)*BR_hjhihi(j,i)*BR_hjtautau(i)**2.0D0
#endif
#ifdef LEPh2h1_4b
	   chann=n+(Dneut_id(nLEPh2h1_4b) -1)*NHIGGS**2+offset
	   R_sig_BR(chann)=CS_lep_hjhi_ratio(j,i)*BR_hjbb(j)*BR_hjbb(i)	 	     
#endif
#ifdef LEPh2h1_4tau
	   chann=n+(Dneut_id(nLEPh2h1_4tau) -1)*NHIGGS**2+offset
	   R_sig_BR(chann)=CS_lep_hjhi_ratio(j,i)*BR_hjtautau(j)*BR_hjtautau(i)	
#endif
#ifdef LEPh2h1_h1h1h1_6b
	   chann=n+(Dneut_id(nLEPh2h1_h1h1h1_6b) -1)*NHIGGS**2+offset	   
	   R_sig_BR(chann)=CS_lep_hjhi_ratio(j,i)*BR_hjhihi(j,i)*BR_hjbb(i)**3.0D0	 	   
#endif
#ifdef LEPh2h1_h1h1h1_6tau
	   chann=n+(Dneut_id(nLEPh2h1_h1h1h1_6tau) -1)*NHIGGS**2+offset	   
	   R_sig_BR(chann)=CS_lep_hjhi_ratio(j,i)*BR_hjhihi(j,i)*BR_hjtautau(i)**3.0D0	
#endif
#ifdef LEPh2Z_h1h1Z_2b2tau
	   chann=n+(Dneut_id(nLEPh2Z_h1h1Z_2b2tau) -1)*NHIGGS**2+offset	   
	   R_sig_BR(chann)=CS_lep_hjZ_ratio(j)*BR_hjhihi(j,i)*BR_hjbb(i)*BR_hjtautau(i)
#endif
#ifdef LEPh2h1_2b2tau
	   chann=n+(Dneut_id(nLEPh2h1_2b2tau) -1)*NHIGGS**2+offset
	   R_sig_BR(chann)=CS_lep_hjhi_ratio(j,i)*BR_hjbb(j)*BR_hjtautau(i)	   	 
#endif
#ifdef LEPh2h1_2tau2b
	   chann=n+(Dneut_id(nLEPh2h1_2tau2b) -1)*NHIGGS**2+offset
	   R_sig_BR(chann)=CS_lep_hjhi_ratio(j,i)*BR_hjtautau(j)*BR_hjbb(i)		   
#endif
	  enddo	               	  
	  endif
	
	  allowed = .TRUE.



! b) Determine the average Higgs mass for the Higgs combinations
!     (so far only done for the single Higgs channels)
!	/!\: currently Mh_avg is not used and also not needed(!) 
!     	     for delta_Mh_LEP of the order of 1-2 GeV or below
!   - b1: single neutral Higgs channels
	do m=0,nLEP_NSneutCHN-1,1
	  do n=1,NHIGGS
	    baseindex=NHIGGS*m
	    sim_mass_sum(n+baseindex)=1
            higgs_combination(n+baseindex,n)=.TRUE.
	    Mh_avg(n+baseindex)=Mh(n)
	    do i=1,NHIGGS
              if(i .ne. n) then
	        if( (Mh(i) .ge. Mh(n)) .and. ( (Mh(i)-Mh(n)) .le. delta_Mh )
     &	 	  .and. ( R_sig_BR_sing(i+baseindex)*R_sig_BR_sing(n+baseindex) .ne. 0d0)
     &		    .and.  table_applicable(i+baseindex) ) then
	          Mh_avg(n+baseindex)=Mh_avg(n+baseindex)+Mh(i)
                  sim_mass_sum(n+baseindex)=sim_mass_sum(n+baseindex)+1
	          higgs_combination(n+baseindex,i)=.TRUE.
	        endif
	      endif
	    enddo
            Mh_avg(n+baseindex)=Mh_avg(n+baseindex)/dble(sim_mass_sum(n+baseindex))
          enddo
	enddo
!   - b2: single charged Higgs channels
	do m=0,nLEP_NScharCHN-1,1
	  do n=1,NHPLUS
	    baseindex=NHPLUS*m+nLEP_NSneutCHN*NHIGGS
	    sim_mass_sum(n+baseindex)=1
            higgs_combination(n+baseindex,n)=.TRUE.
	    Mh_avg(n+baseindex)=MHplus(n)
	    do i=1,NHPLUS
              if(i .ne. n) then
	      if( (MHplus(i) .ge. MHplus(n)) .and. ( (MHplus(i)-MHplus(n)) .le. delta_Mhp) 
     &	 	  .and. ( R_sig_BR_sing(i+baseindex)*R_sig_BR_sing(n+baseindex) .ne. 0d0) 
     &		    .and.  table_applicable(i+baseindex) ) then
	          Mh_avg(n+baseindex)=Mh_avg(n+baseindex)+MHplus(i)
                  sim_mass_sum(n+baseindex)=sim_mass_sum(n+baseindex)+1
	          higgs_combination(n+baseindex,i)=.TRUE.
	        endif
	      endif
	    enddo
            Mh_avg(n+baseindex)=Mh_avg(n+baseindex)/dble(sim_mass_sum(n+baseindex))
          enddo
	enddo


c>> check
c          offset = nLEP_NSneutCHN*NHIGGS
c          do n=1,NHPLUS
c		do baseindex=offset,offset+NHPLUS*(nLEP_NScharCHN-1),NHPLUS
c	  	write(*,*)'n=',n
c                write(*,*)'Mh_avg(',n+baseindex,')=',Mh_avg(n+baseindex)
c	  	write(*,*)'R_sig_BR(',n+baseindex,')=',R_sig_BR(n+baseindex)
c		enddo
c	  enddo
c<<

! no Higgs mass combination done for the double Higgs channels 
	do m=0,nLEP_NDCHN-1,1
	  do n=1,NHIGGS**2
	    sim_mass_sum(n+m*NHIGGS**2+nLEP_NSneutCHN*NHIGGS+nLEP_NScharCHN*NHPLUS)=1
	  enddo				
	enddo




! comparing input to the predicted LEP results	   	  		      
	     predicted_limit = .TRUE.

!   single neutral Higgs analyses
	   do n=1,NHIGGS
	    do k=0,nLEP_NSneutCHN-1,1 	   
	     runid=n+NHIGGS*k
	     call check_LEP_bound1(Mh_avg(runid), 
     &	      R_sig_BR(runid),predratio(runid),k+1,predicted_limit)
 	    enddo
	   enddo


!   single charged Higgs analyses
	   do n=1,NHPLUS
	    do k=0,nLEP_NScharCHN-1,1 	   
	     runid=n+NHPLUS*k+nLEP_NSneutCHN*NHIGGS
	     call check_LEP_bound1(Mh_avg(runid), 
     &	      R_sig_BR(runid),predratio(runid),nLEP_NSneutCHN+k+1,predicted_limit)
 	    enddo
	   enddo


           if(.not. useSingH) then
           do n=1,NHIGGS**2	 	   
            i=findi(NHIGGS,n)
            j=findj(NHIGGS,n)

	    do k=0,nLEP_NDCHN-1,1
	     runid=n+k*NHIGGS**2+nLEP_NSneutCHN*NHIGGS+nLEP_NScharCHN*NHPLUS
	     call check_LEP_bound2(Mh(i),Mh(j),R_sig_BR(runid),predratio(runid),
     &	      k+1,predicted_limit)	    

	    enddo 	    
           enddo	  		  
	   endif

! find out which channel is best for comparing with experimental result	  

        if(useSingH) then
	  number_LEP_channels = nLEP_NSneutCHN*NHIGGS+nLEP_NScharCHN*NHPLUS
	else
	  number_LEP_channels =	nLEP_NSneutCHN*NHIGGS+nLEP_NScharCHN*NHPLUS
     &			       +nLEP_NDCHN*NHIGGS**2
	endif

#ifdef DEBUG
	do nhelp=1,number_LEP_channels,1
	    call determine_output_channel_number(1,nhelp,output_chan)
	    call write_output_channel(6,NHIGGS,NHPLUS,whichexpt,output_chan)
	    write(*,*)'   R_sig_BR=sig*BR/sig*BR_ref =',R_sig_BR(nhelp)
            write(*,*)'   R_sig_BR/S95_pred.         =',predratio(nhelp) 
	enddo
#endif

	  mlratio=my_maxloc(predratio,number_LEP_channels)

! Determine the reaction number n connected to mlratio.
! For mlratio <= nLEP_NSCHN*NHIGGS there are NHIGGS reactions per table, 
! i.e. n is identical with the Higgs species.
! For mlratio >  nLEP_NSCHN*NHIGGS there are NHIGGS**2-NHIGGS reactions per table,
! i.e. n lies between 1 and NHIGGS**2
	
	n=LEP_reaction_number(NHIGGS,NHPLUS,mlratio)

! Determine the analysis number mlindex connected to mlratio. 
!(see LEP_analysis_number(..) for explanation)
	mlindex=LEP_analysis_number(NHIGGS,NHPLUS,mlratio)

! comparing R_sig_BR(mlratio) with observed S95 value
	predicted_limit = .FALSE.

	if(predratio(mlratio) .gt. 0d0) then
	  if((mlratio .le. nLEP_NSneutCHN*NHIGGS+nLEP_NScharCHN*NHPLUS))then
	    call check_LEP_bound1(Mh_avg(mlratio),R_sig_BR(mlratio),
     &				  obsratio,mlindex,predicted_limit)
	  else
	    i=findi(NHIGGS,n)	      
	    j=findj(NHIGGS,n)
	    mlindex=mlindex-nLEP_NSneutCHN-nLEP_NScharCHN
	    call check_LEP_bound2(Mh(i),Mh(j), 
     &	     R_sig_BR(mlratio),obsratio,mlindex,predicted_limit)
	  endif
	else
	  obsratio = 0d0
	endif

          if(obsratio .lt. 1d0) then
! 		obsratio = 0d0 is considered "allowed". 
! 		This is where the tables give no exclusion, e.g. 
! 		because the Higgs mass is out of range.
	   allowed=.True.
	   result_flag=1
	  else
	   allowed=.False.
	   result_flag=0
	  endif

	model_R_sig_BR=R_sig_BR(mlratio)
	most_sens_predratio=predratio(mlratio)
	num_H_combined=sim_mass_sum(mlratio)
	do i=1,NHIGGS
		final_h_comb(i)=higgs_combination(mlratio,i)
	enddo
#ifdef VERBOSE
	write(*,*)'LEP channel with highest sensitivity:'
	    call determine_output_channel_number(1,mlratio,output_chan)
	    call write_output_channel(6,NHIGGS,NHPLUS,whichexpt,output_chan)

	write(*,*)'   R_sig_BR/S95_predicted =',most_sens_predratio
	write(*,*)'   R_sig_BR/S95_observed  =',obsratio
#endif

	end



!******************************************************************
	subroutine check_LEP_applicability(table_applicable, tablenum, higgsnum)
!******************************************************************
	implicit none
	logical table_applicable,CPeven,CPodd
	integer higgsnum,tablenum,index

#include "input_cb.h"
#include "tables-indexing-lep_cb.h"

#ifdef DEBUG
	write(*,*)'LEP applicability check, Mh=',Mh
#endif

	if(tablenum .le. nLEP_NSneutCHN) then
	  index = id_Sneut(tablenum)

  	  if((index .eq. nDELPHIbbh_4b) 
     &		.or. (index .eq. nDELPHIbbh_2b2tau) 
     &		.or. (index .eq. nDELPHItautauh_4tau) 
     &          .or. (index .eq. nOPALbbh_2b2tau)) then
	    if(CPeven(CP_value(higgsnum))) then
		table_applicable = .TRUE.
	    else
		table_applicable = .FALSE.
	    endif
	  elseif((index .eq. nDELPHIbba_4b) 
     &          .or. (index .eq. nDELPHIbba_2b2tau) 
     &          .or. (index .eq. nDELPHItautaua_4tau) 
     &          .or. (index .eq. nOPALbba_2b2tau)) then
	    if(CPodd(CP_value(higgsnum))) then
		table_applicable = .TRUE.
	    else
		table_applicable = .FALSE.
	    endif
	  else
		table_applicable = .TRUE.
	  endif
	endif

	if((tablenum .gt. nLEP_NSneutCHN)
     &		.and. (tablenum .le. nLEP_NSneutCHN+nLEP_NScharCHN)) then
	  index = id_Schar(tablenum-nLEP_NSneutCHN)

c ... enter applicability checks here

	  table_applicable = .TRUE.
	endif


#ifdef DEBUG
	if(table_applicable) then	
 	  write(*,*)'table ',tablenum,' applies to model point'
 	else
 	  write(*,*)'table ',tablenum,' does not apply to model point'
	endif
#endif

	end



!******************************************************************
	integer function findj(NHIGGS,n)
!******************************************************************
	implicit none
	integer n,NHIGGS

	findj=int((n-1)/NHIGGS)+1
	
	end


!******************************************************************
	integer function findi(NHIGGS,n)
!******************************************************************
	implicit none
	integer n,NHIGGS

	findi=n-NHIGGS*int((n-1)/NHIGGS)		

        end


!******************************************************************
	integer function LEP_reaction_number(NHIGGS,NHPLUS,channel)
!******************************************************************
!remark: gives the reaction number within the same analysis, i.e.
!	 a number between 1,..,NHIGGS for single Higgs analyses
!	 and      between 1,..,NHIGGS**2 for double Higgs analyses
	implicit none
	integer channel,NHIGGS,NHPLUS,chn_wo_offset
#include "tables-indexing-lep_cb.h"

	if(channel .le. nLEP_NSneutCHN*NHIGGS) then
	  LEP_reaction_number=channel-int((channel-1)/NHIGGS)*NHIGGS
	elseif((channel .gt. nLEP_NSneutCHN*NHIGGS) 
     & 		.and. (channel .le. nLEP_NSneutCHN*NHIGGS+nLEP_NScharCHN*NHPLUS)) then
	  chn_wo_offset = channel-nLEP_NSneutCHN*NHIGGS
	  LEP_reaction_number=chn_wo_offset-int((chn_wo_offset-1)/NHPLUS)*NHPLUS
	else
	  chn_wo_offset = channel-nLEP_NSneutCHN*NHIGGS-nLEP_NScharCHN*NHPLUS
	  LEP_reaction_number=chn_wo_offset-int((chn_wo_offset-1)/NHIGGS**2)*NHIGGS**2
	endif

	end

!******************************************************************
        integer function LEP_analysis_number(NHIGGS,NHPLUS,channel)
!******************************************************************
        implicit none
        integer channel,NHIGGS,NHPLUS
#include "tables-indexing-lep_cb.h"

	if(channel .le. nLEP_NSneutCHN*NHIGGS) then 
          LEP_analysis_number=1+int((channel-1)/NHIGGS)

        elseif((channel .gt. nLEP_NSneutCHN*NHIGGS)
     &         .and. (channel .le. nLEP_NSneutCHN*NHIGGS+nLEP_NScharCHN*NHPLUS)) then
          LEP_analysis_number=
     &		1+int((channel-nLEP_NSneutCHN*NHIGGS-1)/NHPLUS)+nLEP_NSneutCHN   

        elseif(channel .gt. nLEP_NSneutCHN*NHIGGS+nLEP_NScharCHN*NHPLUS) then
          LEP_analysis_number=
     &	  	1+int((channel-nLEP_NSneutCHN*NHIGGS-nLEP_NScharCHN*NHPLUS-1)/NHIGGS**2)
     &          +nLEP_NSneutCHN+nLEP_NScharCHN
        endif

	end
	


!******************************************************************
	subroutine check_LEP_bound1(Mh1, 
     &	  factBR,ratio,selec,pred)
!******************************************************************
! remark: selec is expected to run from 1 to nLEP_NSneutCHN+nLEP_NScharCHN=nLEP_NSCHN
	implicit none
	logical pred
	integer tabsize,pred_obs_flag
	integer i,ilow,ihigh,selec,Sid
	double precision Mh1,Mh1_test,Mh1_below,factBR,ratio
        double precision S95_above,S95_below,S95_interpol

#include "tables-indexing-lep_cb.h"
#include "tables-data-lep_cb.h"


	if(selec .le. nLEP_NSneutCHN) then
	  Sid=Sid_Sneut(selec)
	else
	  Sid=Sid_Schar(selec-nLEP_NSneutCHN)
	endif

	if(factBR .eq. 0d0) then
	ratio=0d0
	return
	endif

	if(pred) then
	  pred_obs_flag=1
	else
	  pred_obs_flag=2
	endif

        ratio=0.0D0
	tabsize=nint((Mhmax(Sid)-Mhmin(Sid))/sep(Sid))+1

	
! check if mass is within mass range of table:
	if(Mh1 .gt. Mhmax(Sid)) then
	  ratio=0D0
	elseif(Mh1 .lt. Mhmin(Sid)) then	 
	  ratio=0D0
	else	
! linear interpolate between points in table
          ilow=int((Mh1-Mhmin(Sid))/sep(Sid))+1

! check if interpolation is needed 
!	The expression in the following if-statement should be mathematically zero.
!	However, numerically, it is sometimes only nearly zero.
	  if( ((Mh1-Mhmin(Sid))/sep(Sid)+1d0-dble(ilow)) .lt. 1d-7) then
		S95_interpol=S95_1d(pred_obs_flag,Sid,ilow)
	  else
	    i=ilow	 	 
	    Mh1_below=dble(i-1)*sep(Sid)+Mhmin(Sid)
	    S95_below=S95_1d(pred_obs_flag,Sid,i)
	    S95_above=S95_1d(pred_obs_flag,Sid,i+1)
     	    S95_interpol=S95_below +(S95_above-S95_below)/sep(Sid)*(Mh1-Mh1_below) 
	  endif
! factBR is the ratio (sigma*BRs)_model/(sigma*BRs)_ref.
! factBR for the channel with the highest factBR/S95_interpol_predicted
! has to be smaller then S95_interpol_observed in order to yield an
! allowed scenario. 
! The latter is equivalent to ratio=factBR/S95_interpol_observed < 1.
	  ratio=factBR/S95_interpol		

	endif 
	
	end



!******************************************************************
	subroutine check_LEP_bound2(Mh1in,Mh2in, 
     &	  factBR,ratio,selec,pred)
!******************************************************************	  
! remark: selec is expected to run from 1 to nLEP_NDCHN
	implicit none
	logical pred

	integer imax,jmax,selec,pred_obs_flag,index
	integer i,j
	integer ilow,jlow,ihigh,jhigh,ii,jj,k
	double precision Mh1in,Mh2in,Mh1,Mh2,ratio,Mhtemp
        double precision factBR,rvec(3),testtri
        double precision Mh1bit,Mh2bit,c1,c2,c3	

#include "tables-indexing-lep_cb.h"
#include "tables-data-lep_cb.h"

		
	if(factBR .eq. 0d0) then
	ratio=0d0
	return
	endif

	if(pred) then
	  pred_obs_flag=1
	else
	  pred_obs_flag=2
	endif

	imax=int((Mh2max(selec)-Mh2min(selec)))/sep2(selec)+1
	jmax=int((Mh1max(selec)-Mh1min(selec)))/sep1(selec)+1	


!>> Ensure that Mh2 > Mh1 in order to make proper use of some 
!   of the tables.
!
!* tables 18 and 19:
!------------------
! table 18: process e+ e- -> Hi Hj with Hi, Hj -> b b
! table 19: process e+ e- -> Hi Hj with Hi, Hj -> tau tau
! Both cross sections are symmetric under the exchange m_Hi <--> m_Hj . 
! However, both tables are only given for Mh2 >= Mh1. 
! /!\: not anymore: >>Therefore, we swap the masses if Mh2in < Mh1in.<<
! Both permuatations of Higgs bosons will be checked anyway.
!  
!* tables 23 and 24:
!-------------------
! table 23: process e+ e- -> Hi Hj with, Hi -> b b, Hj -> tau tau 
!           and m_Hi <= m_Hj, i.e. Mh2 >= Mh1. 
!		Thus Mh2 -> m_Hj
!		     Mh1 -> m_Hi
! table 24: process e+ e- -> Hi Hj with, Hi -> tau tau, Hj -> b b
!           and m_Hi <= m_Hj, i.e. Mh2 >= Mh1.
!		Thus Mh2 -> m_Hj
!		     Mh1 -> m_Hi
! The external loop over n=1 ... NHIGSS**2 leads with 
!	i=findi(NHIGGS,n) and j=findj(NHIGGS,n) to a sum over all combinations
!	(i,j) in {(1,1), (1,2), (2,1), (2,2), (1,3), (3,1), (2,3), (3,2), ...}
! Thus in total one checks all possibilities
! for e+ e- -> Hi Hj with, Hi -> b b, Hj -> tau tau for all masses 
! m_Hi and m_Hj. Therefore, no swapping of masses is necessary.
!

	 Mh1=Mh1in
	 Mh2=Mh2in

	rvec(3)=0d0
	      	
! fill S95 from file
! row 0 and column 0 in LEP file contain higgs masses 
! and (0,0) ie top left set to -100	

! check if mass is within mass range of table:
	if(Mh1 .lt. Mh1min(selec)) then 
	  ratio=0D0
	  return
	endif
	if(Mh1 .gt. Mh1max(selec)) then
	  ratio=0D0
          return
        endif
	if(Mh2 .lt. Mh2min(selec)) then
	  ratio=0D0
          return
        endif
	if(Mh2 .gt. Mh2max(selec)) then
	  ratio=0D0
          return
        endif

! if hi->hj+hj is needed, check it's kinematically possible:         
	index=id_Dneut(selec)
	if((index .eq. nLEPh2Z_h1h1Z_4bZ) .or. (index .eq. nLEPh2Z_h1h1Z_4tauZ) 
     &		.or. (index .eq. nLEPh2Z_h1h1Z_2b2tau) 
     &		.or. (index .eq. nLEPh2h1_h1h1h1_6b) 
     &		.or. (index .eq. nLEPh2h1_h1h1h1_6tau)) then
	  if(Mh2 .lt. 2d0*Mh1) then
	    ratio=0d0
	    return
	  endif
	endif
	 		
! Interpolation: 
! points in table are treated as corners of flat triangular surfaces 	

	 
	 jlow=1+int((Mh1-Mh1min(selec))/sep1(selec))
	 Mh1bit=(Mh1-Mh1min(selec))/sep1(selec)-dble(jlow-1)
  	 	 	 
	 ilow=1+int((Mh2-Mh2min(selec))/sep2(selec))
	 Mh2bit=(Mh2-Mh2min(selec))/sep2(selec)-dble(ilow-1)	          			


! check if interpolation is needed, 
! (if  Mh1bit=Mh2bit=0 (or almost) no interpolation is needed)

	 if((Mh1bit**2+Mh2bit**2) .le. 1d-7) then !tag 1
		rvec(3)=S95_2d(pred_obs_flag,selec,ilow,jlow)
	 else !tag 1
	  
!      *******    find out which triangle required point r is in
!      * B * *    i.e. triangle of orientation 'A' or 'B'
!      * * A *
!      *******	    	    
	   
	 testtri=Mh2bit/sqrt(Mh1bit**2.0D0+Mh2bit**2.0D0)
   	  	  
	  if(testtri.gt.0.7071D0)then !triangle orientation A
! check that all of the corners of the triangle are positive	 
	     if(S95_2d(pred_obs_flag,selec,ilow+1,jlow).lt.0.0D0)then
	      rvec(3)=-6.0D0	  	   
             elseif(S95_2d(pred_obs_flag,selec,ilow+1,jlow+1).lt.0.0D0)then
	      rvec(3)=-6.0D0	   
             elseif(S95_2d(pred_obs_flag,selec,ilow,jlow).lt.0.0D0)then
	      rvec(3)=-6.0D0 
	     else
! find 'height' (i.e. S95 value) of each corner of triangle
	   
!             c3*       
!               * *          orth sides have length=1
!  rvec(2)^     * r * 
!         |     *     *      rvec(3) is S95_interpol
!         v   c1*********c2
!               <->
!             rvec(1)   
	   
	      c1=S95_2d(pred_obs_flag,selec,ilow+1,jlow)
	      c2=S95_2d(pred_obs_flag,selec,ilow+1,jlow+1)
	      c3=S95_2d(pred_obs_flag,selec,ilow,jlow)
! find position of r relative to corner c1	    
	      rvec(1)=Mh1bit
	      rvec(2)=1.0D0-Mh2bit
! find 'height' of triangle at point r      
	      rvec(3)=rvec(1)*(c2-c1)+rvec(2)*(c3-c1)+c1
	    	    	    
	     endif
	  else !triangle orientation B
! check that all of the corners of the triangle are positive	 
	     if(S95_2d(pred_obs_flag,selec,ilow,jlow+1).lt.0.0D0)then
	      rvec(3)=-6.0D0	  	   
             elseif(S95_2d(pred_obs_flag,selec,ilow+1,jlow+1).lt.0.0D0)then
	      rvec(3)=-6.0D0	   
             elseif(S95_2d(pred_obs_flag,selec,ilow,jlow).lt.0.0D0)then
	      rvec(3)=-6.0D0 
	     else
! find 'height' (i.e. S95 value) of each corner of triangle	   
	      c1=S95_2d(pred_obs_flag,selec,ilow,jlow+1)
	      c2=S95_2d(pred_obs_flag,selec,ilow,jlow)
	      c3=S95_2d(pred_obs_flag,selec,ilow+1,jlow+1)
! find position of r relative to corner c1	    
	      rvec(1)=1.0D0-Mh1bit
	      rvec(2)=Mh2bit
! find 'height' of triangle at point r	      
	      rvec(3)=rvec(1)*(c2-c1)+rvec(2)*(c3-c1)+c1
	     endif 	   	 	  
	  endif	!triangle orientation
	 endif !tag 1
	
	 if(rvec(3) .lt. 0d0)then
	   ratio = 0D0    
	 else
	   ratio = factBR/rvec(3)
	 endif

	end

