! This file is part of HiggsBounds
!  -KW
!****************************************************** 
module theory_XS_SM_functions 
!****************************************************** 
 ! XS_qq_HW_SM,XS_qq_HZ_SM,XS_gg_H_SM,XS_bb_H_SM,XS_vbf:_H_SM
 ! in units of fb
 ! functions are fitted to data 
 ! downloaded from http://maltoni.home.cern.ch/maltoni/TeV4LHC/bbh-tev_ed.dat
 ! on Thursday 10th April 2008
 ! data has range 100 to 300 GeV
 !
 ! functions XS_bg_Hb_SM,XS_bg_Hb_c1_SM,XS_bg_Hb_c2_SM were generated by Oliver Brein
 
 double precision :: tevXS_SM_functions_xmin=60.0D0
 double precision :: tevXS_SM_functions_xmax=335.0D0
! this must be narrower than range of 
!XS_bg_Hb_SM,XS_bg_Hb_c1_SM,XS_bg_Hb_c2_SM ([10:400])
!theory_tevSfunctions (currently [50:400])

 contains 

!files from OB (his calculations)
!only change I made:
! end->end function
#include "cs-ratios_sigma-bg-Hb/Tevatron.sigma_bg_Hb.h"
#include "cs-ratios_sigma-bg-Hb/Tevatron.sigma_bg_Hb.ptmin15.etamax2.h"
#include "cs-ratios_sigma-bg-Hb/Tevatron.sigma_bg_Hb.ptmin20.etamax2.5.h"
#include "cs-ratios_sigma-bg-Hb/Tevatron.sigma_bg_Hb.ptmin15.etamax2.5.h"

 !****************************************************** 
 subroutine check_range(Mh,rangeok,badvalue)
 !****************************************************** 
  implicit none
  double precision, intent(in) :: Mh 
  logical :: rangeok
  double precision :: badvalue

  badvalue=-1.0D0

  if(    Mh.lt.tevXS_SM_functions_xmin)then
   write(*,*)'Warning: Higgs mass is below valid range for the SM cross section functions' 
   rangeok=.False.
  elseif(Mh.gt.tevXS_SM_functions_xmax)then
   write(*,*)'Warning: Higgs mass is above valid range for the SM cross section functions' 
   rangeok=.False.
  elseif( .not.(Mh.gt.0.0D0) )then !in case Mh is NaN
   write(*,*) 'Warning: Higgs mass not in valid range for the SM cross section functions'  
   rangeok=.False. 
  else
   rangeok=.True.
  endif 

 end subroutine check_range

 !****************************************************** 
 function XS_bg_Hb_SM(x)
 !******************************************************
  implicit none
  double precision :: XS_bg_Hb_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue

  call check_range(x,rangeok,badvalue)
  if(rangeok)then
    func=1.0D3*tev_cs_bg_Hb_SM(x) !convert from pb to fb
  else
    func=badvalue
  endif

  XS_bg_Hb_SM=func

 end function XS_bg_Hb_SM 
 !****************************************************** 
 function XS_bg_Hb_c1_SM(x)
 !******************************************************
  implicit none
  double precision :: XS_bg_Hb_c1_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue

  call check_range(x,rangeok,badvalue)
  if(rangeok)then
    func=1.0D3*tev_cs_bg_Hb_c1_SM(x) !convert from pb to fb
  else
    func=badvalue
  endif

  XS_bg_Hb_c1_SM=func
  
 end function XS_bg_Hb_c1_SM 
 !****************************************************** 
 function XS_bg_Hb_c2_SM(x)
 !******************************************************
  implicit none
  double precision :: XS_bg_Hb_c2_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue

  call check_range(x,rangeok,badvalue)
  if(rangeok)then
    func=1.0D3*tev_cs_bg_Hb_c2_SM(x) !convert from pb to fb
  else
    func=badvalue
  endif

  XS_bg_Hb_c2_SM= func
 
 end function XS_bg_Hb_c2_SM 
 !****************************************************** 
 function XS_bg_Hb_c3_SM(x)
 !******************************************************
  implicit none
  double precision :: XS_bg_Hb_c3_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue

  call check_range(x,rangeok,badvalue)
  if(rangeok)then
    func=1.0D3*SMCS_tev_pp_bg_Hb_c3(x) !convert from pb to fb
  else
    func=badvalue
  endif

  XS_bg_Hb_c3_SM= func
 
 end function XS_bg_Hb_c3_SM 
 !****************************************************** 
 function XS_qq_HW_SM(x)
 !******************************************************
  implicit none
  double precision :: XS_qq_HW_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2
  
  a0 = 5.7514105496046D0
  a0p5 = -0.375021739545092D0
  a1 = 0.0049451487167627D0
  a2 = -3.77008582179264D-06
  
  call check_range(x,rangeok,badvalue)
  if(rangeok)then
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
  else
    func=badvalue
  endif

  XS_qq_HW_SM=func

 end function XS_qq_HW_SM
 !****************************************************** 
 function XS_qq_HZ_SM(x)
 !****************************************************** 
  implicit none
  double precision :: XS_qq_HZ_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2
 
  a0 = 5.29935340004443D0
  a0p5 = -0.351677660532052D0
  a1 = 0.0047848452802514D0
  a2 = -3.82425969474559D-06 

  call check_range(x,rangeok,badvalue)
  if(rangeok)then
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
  else
    func=badvalue
  endif

  XS_qq_HZ_SM=func

 end function XS_qq_HZ_SM 
 !****************************************************** 
 function XS_gg_H_SM(x)
 !******************************************************
  implicit none
  double precision :: XS_gg_H_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2
 
  a0 = 5.59682783597183D0
  a0p5 = -0.244216706673437D0
  a1 = 0.000365613425058581D0
  a2 = 2.66122261164927D-06  
	
  call check_range(x,rangeok,badvalue)
  if(rangeok)then
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
  else
    func=badvalue
  endif

  XS_gg_H_SM=  func

 end function XS_gg_H_SM
 !****************************************************** 
 function XS_gg_H_SM_9713(x)
 !******************************************************
 ! from CDF note 9713, D0 Note 5889
 ! note: need to check valdity outside 100<x<200

  implicit none
  double precision :: XS_gg_H_SM_9713
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2
 
  a0 = 5.85040646662332D0
  a0p5 = -0.289955377784879D0
  a1 = 0.00367931930924978D0
  a2 =  -5.18285806787161D-06 

  call check_range(x,rangeok,badvalue)
  if((x.lt.100.0D0).or.(x.gt.200.0D0))then
   !stop'function XS_gg_H_SM_9713: need to check validity at this MH'
   write(*,*)'WARNING: function XS_gg_H_SM_9713: need to check validity at this MH'
  endif

  if(rangeok)then
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
  else
    func=badvalue
  endif

  XS_gg_H_SM_9713= func
  
 end function XS_gg_H_SM_9713
 !****************************************************** 
 function XS_gg_H_SM_9674(x)
 !******************************************************
 ! from CDF note 9674
 ! note: need to check valdity outside 100<x<200

  implicit none
  double precision :: XS_gg_H_SM_9674
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2
 
  a0 = 5.46563745525111D0
  a0p5 = -0.25740914779426D0
  a1 = 0.00426592493420152D0
  a2 = -9.15606785912482D-06

  call check_range(x,rangeok,badvalue)
  if((x.lt.100.0D0).or.(x.gt.200.0D0))then
   !stop'function XS_gg_H_SM_9674: need to check validity at this MH'
   write(*,*)'WARNING: function XS_gg_H_SM_9674: need to check validity at this MH'
  endif

  if(rangeok)then
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
  else
    func=badvalue
  endif

  XS_gg_H_SM_9674=  func
 
 end function XS_gg_H_SM_9674
 !****************************************************** 
 function XS_bb_H_SM(x)
 !******************************************************
  implicit none
  double precision :: XS_bb_H_SM
  double precision, intent(in) :: x
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2
 
  a0 = 5.41583328209568D0
  a0p5 = -0.453323023285831D0
  a1 = 0.00514220061294974D0
  a2 = -3.31488355831377D-06
	
  call check_range(x,rangeok,badvalue)
  if(rangeok)then
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
  else
    func=badvalue
  endif

  XS_bb_H_SM=  func

 end function XS_bb_H_SM 
 !****************************************************** 
 function XS_vbf_SM(x)
 !******************************************************
  implicit none
  double precision :: XS_vbf_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2
 
  a0 = 3.03330688577339D0
  a0p5 = -0.061167148821396D0
  a1 = -0.00424395931917914D0
  a2 = 7.67964289500027D-08
	
  call check_range(x,rangeok,badvalue)
  if(rangeok)then
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
  else
    func=badvalue
  endif

  XS_vbf_SM= func

 end function XS_vbf_SM
 !****************************************************** 
 function XS_ttH_SM(x)
 !******************************************************
  implicit none
  double precision :: XS_ttH_SM
  double precision, intent(in) :: x 
  logical :: rangeok
  double precision :: func,badvalue
  double precision :: a0,a0p5,a1,a2
 	
  a0 = 1.23241191811843D0
  a0p5 = 0.188533306634442D0
  a1 = -0.0240486573108739D0
  a2 = 1.90098085776576D-05

  call check_range(x,rangeok,badvalue)
  if(rangeok)then
    func=10.0D0**(a0+a0p5*x**0.5D0+a1*x+a2*x**2.0D0)
  else
    func=badvalue
  endif

  XS_ttH_SM=  func

 end function XS_ttH_SM
 !******************************************************
 function LEPHiggsstrahlung_XS_SM(sqrts, MH)
 !using eq.4.18 in arXiv:hep-ph/0503172 (Abdelhak Djouadi)
 !*****************************************************
 use usefulbits, only : MZ,MW,GF,pi
  implicit none
  double precision :: LEPHiggsstrahlung_XS_SM
  double precision, intent(in) :: sqrts, MH
  double precision :: ae,ve !Z charges of the electron
  double precision :: SW2
  double precision :: lambda !two-particle phase space function
  double precision :: coeff
  double precision :: s,MW2,MZ2,MH2

  s   = sqrts**2.0D0
  MH2 = MH**2.0D0
  MW2 = MW**2.0D0
  MZ2 = MZ**2.0D0
  SW2 = 1.0D0-MW2/MZ2

  ae=-1.0D0
  ve=-1.0D0+4.0D0*SW2

  lambda=(1.0D0 - MH2/s - MZ2/s )**2.0D0 &
        & - 4.0D0*MH2*MZ2/s**2.0D0

  coeff=GF**2.0D0*MZ**4.0D0/(96.0D0*pi) * (ae**2.0D0 + ve**2.0D0)

  LEPHiggsstrahlung_XS_SM= coeff /s *sqrt(lambda) &
           & * ( lambda + 12.0D0 * MZ2/s ) &
           & / ( 1.0d0  -  MZ2/s  )**2.0D0

 end function LEPHiggsstrahlung_XS_SM
 !******************************************************
end module theory_XS_SM_functions
